//! @file CuiPlot.cc
//! @brief CuiPlotクラス
//!
//! CUI(ランレベル3)の画面上にグラフの描画を行います。ただし，FrameGraphicsクラス が必要。
//!
//! @date 2019/03/29
//! @author Yuki YOKOKURA
//
// Copyright (C) 2011-2019 Yuki YOKOKURA
// This program is free software;
// you can redistribute it and/or modify it under the terms of the FreeBSD License.
// For details, see the License.txt file.

#include "CuiPlot.hh"
#include "ARCSeventlog.hh"

using namespace ARCS;

//! @brief コンストラクタ
CuiPlot::CuiPlot(FrameGraphics& FG, int x, int y, int w, int h)
	: pFG(&FG),
	  AxisColor(FrameGraphics::WHITE),
	  GridColor(FrameGraphics::GRAY25),
	  BackColor(FrameGraphics::BLACK),
	  TextColor(FrameGraphics::WHITE),
	  ScrLeft(x),
	  ScrTop(y),
	  ScrWidth(w),
	  ScrHeight(h),
	  Xmax(1),
	  Xmin(-1),
	  Ymax(1),
	  Ymin(-1),
	  XgridNum(4),
	  Xgrid(0),
	  YgridNum(4),
	  Ygrid(0),
	  ScaleLen(5),
	  XLabelMargin(5),
	  YLabelMargin(5),
	  MarginWidth(30),
	  MarginHeight(25),
	  XLabel("X Axis Label [unit]"),
	  YLabel("Y Axis Label [unit]"),
	  Xform("%1.1f"),
	  Yform("%1.1f"),
	  NumOfVars(0),
	  LegendNames(),
	  LineColors(),
	  x_prev(0),
	  y_prev(0),
	  var_values_prev(0),
	  IsFirstTime(true),
	  IsVisible(true)
{
	PassedLog();
	
	// 横幅と高さの計算
	ScrWidth  = w - (MarginWidth*2);	// 横幅
	ScrHeight = h - (MarginHeight*2);	// 高さ
	
	// グリッド座標の計算
	XgridCalc();
	YgridCalc();
	
	PassedLog();
}

//! @brief 空コンストラクタ
CuiPlot::CuiPlot()
	: pFG(nullptr),
	  AxisColor(FrameGraphics::WHITE),
	  GridColor(FrameGraphics::GRAY25),
	  BackColor(FrameGraphics::BLACK),
	  TextColor(FrameGraphics::WHITE),
	  ScrLeft(0),
	  ScrTop(0),
	  ScrWidth(0),
	  ScrHeight(0),
	  Xmax(1),
	  Xmin(-1),
	  Ymax(1),
	  Ymin(-1),
	  XgridNum(4),
	  Xgrid(0),
	  YgridNum(4),
	  Ygrid(0),
	  ScaleLen(5),
	  XLabelMargin(5),
	  YLabelMargin(5),
	  MarginWidth(30),
	  MarginHeight(25),
	  XLabel("X Axis Label [unit]"),
	  YLabel("Y Axis Label [unit]"),
	  Xform("%1.1f"),
	  Yform("%1.1f"),
	  NumOfVars(0),
	  LegendNames(),
	  LineColors(),
	  x_prev(0),
	  y_prev(0),
	  var_values_prev(0),
	  IsFirstTime(true),
	  IsVisible(true)
{
	// 横幅と高さの計算
	ScrWidth  = 100 - (MarginWidth*2);	// 横幅
	ScrHeight = 100 - (MarginHeight*2);	// 高さ
	
	// グリッド座標の計算
	XgridCalc();
	YgridCalc();
}

//! @brief ムーブコンストラクタ
CuiPlot::CuiPlot(CuiPlot&& r)
	: pFG(r.pFG),
	  AxisColor(FrameGraphics::WHITE),
	  GridColor(FrameGraphics::GRAY25),
	  BackColor(FrameGraphics::BLACK),
	  TextColor(FrameGraphics::WHITE),
	  ScrLeft(r.ScrLeft),
	  ScrTop(r.ScrTop),
	  ScrWidth(r.ScrWidth),
	  ScrHeight(r.ScrHeight),
	  Xmax(r.Xmax),
	  Xmin(r.Xmin),
	  Ymax(r.Ymax),
	  Ymin(r.Ymin),
	  XgridNum(r.XgridNum),
	  Xgrid(r.Xgrid),
	  YgridNum(r.YgridNum),
	  Ygrid(r.Ygrid),
	  ScaleLen(r.ScaleLen),
	  XLabelMargin(r.XLabelMargin),
	  YLabelMargin(r.YLabelMargin),
	  MarginWidth(r.MarginWidth),
	  MarginHeight(r.MarginHeight),
	  XLabel(r.XLabel),
	  YLabel(r.YLabel),
	  Xform(r.Xform),
	  Yform(r.Yform),
	  NumOfVars(r.NumOfVars),
	  LegendNames(r.LegendNames),
	  LineColors(r.LineColors),
	  x_prev(r.x_prev),
	  y_prev(r.y_prev),
	  var_values_prev(r.var_values_prev),
	  IsFirstTime(r.IsFirstTime),
	  IsVisible(r.IsVisible)
{
	r.pFG = nullptr;
}

//! @brief デストラクタ
CuiPlot::~CuiPlot(){
	PassedLog();
	if(Xgrid)delete[] Xgrid;
	if(Ygrid)delete[] Ygrid;
	if(LegendNames)delete[] LegendNames;
	if(LineColors)delete[] LineColors;
	if(var_values_prev)delete[] var_values_prev;
	PassedLog();
}

void CuiPlot::SetAxisColor(FrameGraphics::FGcolors color){
	// 主軸の色設定
	AxisColor = color;
}

void CuiPlot::SetGridColor(FrameGraphics::FGcolors color){
	// グリッドの色設定
	GridColor = color;
}

void CuiPlot::SetBackColor(FrameGraphics::FGcolors color){
	// 背景色の設定
	BackColor = color;
}

void CuiPlot::SetTextColor(FrameGraphics::FGcolors color){
	// 文字色の設定
	TextColor = color;
	pFG->PrepareFontData(TextColor,BackColor);
}

void CuiPlot::XgridCalc(void){
	// X軸グリッドの計算
	unsigned int i;

	// 動的メモリXgridが確保されていたら削除
	if(Xgrid)delete[] Xgrid;
	Xgrid = new double[XgridNum];	// X軸グリッドX座標
	
	for(i=0;i<XgridNum;i++){
		Xgrid[i]=(Xmax-Xmin)/(double)XgridNum*(double)(i+1) + Xmin;	// X座標の計算
	}
}

void CuiPlot::YgridCalc(void){
	// Y軸グリッドの計算
	unsigned int i;

	// 動的メモリYgridが確保されていたら削除
	if(Ygrid)delete[] Ygrid;
	Ygrid = new double[YgridNum];	// Y軸グリッドY座標
	
	for(i=0;i<YgridNum;i++){
		Ygrid[i]=(Ymax-Ymin)/(double)YgridNum*(double)(i+1) + Ymin;	// Y座標の計算
	}
}

int CuiPlot::XtoScrX(double X) const {
	// X実数値からX画面座標へ変換する関数
	if(X < Xmin)X = Xmin;	// 範囲内だけに描画するようにリミット
	if(Xmax < X)X = Xmax;	// 範囲内だけに描画するようにリミット
	return (int)((double)ScrWidth/(Xmax-Xmin)*(X-Xmin)) + MarginWidth + MarginWidth/2 + ScrLeft;
}

int CuiPlot::YtoScrY(double Y) const {
	// Y実数値からY画面座標へ変換する関数
	if(Y < Ymin)Y = Ymin;	// 範囲内だけに描画するようにリミット
	if(Ymax < Y)Y = Ymax;	// 範囲内だけに描画するようにリミット
	return (int)((double)ScrHeight/(Ymax-Ymin)*(-Y+Ymax)) + 2*MarginHeight/3 + ScrTop;
}

void CuiPlot::DrawAxis(){
	// 軸の描画, グラフの基本描画
	if(IsVisible == false)return;	// 不可視設定なら何もしない
	
	// 背景色で塗りつぶし
	pFG->DrawRectFill(ScrLeft,ScrTop,ScrWidth + (MarginWidth*2),ScrHeight + (MarginHeight*2),BackColor);
	
	DrawGrids();	// グリッドの描画
	DrawLabels();	// ラベルの描画
}

void CuiPlot::DrawGrids(){
	// グリッドの描画
	unsigned int i;
	// X軸グリッドの描画
	for(i=0;i<XgridNum-1;i++){
		pFG->DrawLine(XtoScrX(Xgrid[i]), YtoScrY(Ymin), XtoScrX(Xgrid[i]), YtoScrY(Ymax), GridColor);			// グリッド線の描画
		pFG->DrawLine(XtoScrX(Xgrid[i]), YtoScrY(Ymin), XtoScrX(Xgrid[i]), YtoScrY(Ymin)-ScaleLen, AxisColor);	// 下側目盛線の描画
		pFG->DrawLine(XtoScrX(Xgrid[i]), YtoScrY(Ymax), XtoScrX(Xgrid[i]), YtoScrY(Ymax)+ScaleLen, AxisColor);	// 上側目盛線の描画
	}
	// Y軸グリッドの描画
	for(i=0;i<YgridNum-1;i++){
		pFG->DrawLine(XtoScrX(Xmin), YtoScrY(Ygrid[i]), XtoScrX(Xmax), YtoScrY(Ygrid[i]), GridColor);			// グリッド線の描画
		pFG->DrawLine(XtoScrX(Xmin), YtoScrY(Ygrid[i]), XtoScrX(Xmin)+ScaleLen, YtoScrY(Ygrid[i]), AxisColor);	// 左側目盛線の描画
		pFG->DrawLine(XtoScrX(Xmax), YtoScrY(Ygrid[i]), XtoScrX(Xmax)-ScaleLen, YtoScrY(Ygrid[i]), AxisColor);	// 右側目盛線の描画
	}
	// グラフ外枠の描画
	pFG->DrawRect(MarginWidth + MarginWidth/2 + ScrLeft, 2*MarginHeight/3 + ScrTop, ScrWidth, ScrHeight, AxisColor);
}

void CuiPlot::DrawLabels(){
	// ラベルの描画
	// グリッドラベルの描画
	for(unsigned int i=0;i<XgridNum-1;i++){
		pFG->PrintValue(XtoScrX(Xgrid[i]), YtoScrY(Ymin)+XLabelMargin, FrameGraphics::ALIGN_CENTER, Xform, Xgrid[i]);	// X軸グリッドラベル
	}
	for(unsigned int i=0;i<YgridNum-1;i++){
		pFG->PrintValue(XtoScrX(Xmin)-YLabelMargin, YtoScrY(Ygrid[i]) - LABEL_VERTICAL_ALIGN, FrameGraphics::ALIGN_RIGHT, Yform, Ygrid[i]);	// Y軸グリッドラベル
	}
	// X軸 最小値と最大値ラベルの描画
	pFG->PrintValue(XtoScrX(Xmax), YtoScrY(Ymin)+XLabelMargin, FrameGraphics::ALIGN_CENTER, Xform, Xmax);	// 最大値ラベル
	pFG->PrintValue(XtoScrX(Xmin), YtoScrY(Ymin)+XLabelMargin, FrameGraphics::ALIGN_CENTER, Xform, Xmin);	// 最小値ラベル
	// Y軸 最小値と最大値ラベルの描画
	pFG->PrintValue(XtoScrX(Xmin)-YLabelMargin, YtoScrY(Ymax), FrameGraphics::ALIGN_RIGHT, Yform, Ymax);	// 最大値ラベル	
	pFG->PrintValue(XtoScrX(Xmin)-YLabelMargin, YtoScrY(Ymin) - LABEL_VERTICAL_ALIGN*2, FrameGraphics::ALIGN_RIGHT, Yform, Ymin);	// 最小値ラベル
	// ラベルの描画
	pFG->PrintText(XtoScrX((Xmax-Xmin)/2.0+Xmin), YtoScrY(Ymin) + LABEL_VERTICAL_ALIGN*2 + XLabelMargin*2, FrameGraphics::ALIGN_CENTER, XLabel);	// X軸ラベル
	pFG->PrintText(ScrLeft, ScrTop, FrameGraphics::ALIGN_LEFT, YLabel);	// Y軸ラベル
}

void CuiPlot::SetVariables(const unsigned int var_num, const std::string* var_names, const FrameGraphics::FGcolors* var_colors){
	// プロットする変数の設定
	NumOfVars = var_num;	// 描画する変数の数
	if(MAX_VARIABLE_NUM < NumOfVars)NumOfVars = MAX_VARIABLE_NUM;	// 表示可能な最大数でリミット
	
	// 凡例に表示する変数名の設定
	if(LegendNames)delete[] LegendNames;
	LegendNames = new std::string[NumOfVars];
	for(unsigned int i=0;i<NumOfVars;i++)LegendNames[i] = var_names[i];
	
	// 線の色の設定
	if(LineColors)delete[] LineColors;
	LineColors = new FrameGraphics::FGcolors[NumOfVars];
	for(unsigned int i=0;i<NumOfVars;i++)LineColors[i] = var_colors[i];
	
	// 逐次プロット用 変数値格納変数
	if(var_values_prev)delete[] var_values_prev;
	var_values_prev = new double[NumOfVars];
}

void CuiPlot::DrawLegend(){
	// 凡例の描画
	if(IsVisible == false)return;	// 不可視設定なら何もしない
	unsigned int i;
	for(i=0;i<NumOfVars;i++){
		// 凡例の線の描画
		pFG->DrawLine(
			ScrLeft + LEGEND_LEFT + LEGEND_INTERVAL*i,
			ScrTop + LABEL_VERTICAL_ALIGN + 1,
			ScrLeft + LEGEND_LEFT + LEGEND_INTERVAL*i + LEGEND_LINE_LENGTH,
			ScrTop + LABEL_VERTICAL_ALIGN + 1,
			LineColors[i]
		);
		// 凡例の変数名の描画
		pFG->PrintText(
			ScrLeft + LEGEND_LEFT + LEGEND_INTERVAL*i + LEGEND_LINE_LENGTH + 4,
			ScrTop,
			FrameGraphics::ALIGN_LEFT,
			LegendNames[i]
		);
	}
}

// 関数オーバーロードここから
void CuiPlot::PlotData(double* x, double* y, unsigned int N, FrameGraphics::FGcolors color){
	// データを一括プロットする関数
	if(IsVisible == false)return;	// 不可視設定なら何もしない
	unsigned int i;
	for(i=1;i<N;i++){
		pFG->DrawLine(XtoScrX(x[i-1]), YtoScrY(y[i-1]), XtoScrX(x[i]), YtoScrY(y[i]), color);
	}
}
void CuiPlot::PlotData(double x, double y, FrameGraphics::FGcolors color){
	// データを逐次プロットする関数
	if(IsVisible == false)return;	// 不可視設定なら何もしない
	
	// 最初の1回はプロットしない
	if(IsFirstTime == true){
		IsFirstTime = false;
		x_prev = x;	// 次回用に格納
		y_prev = y;	// 次回用に格納
		return;
	}
	
	pFG->DrawLine(XtoScrX(x_prev),YtoScrY(y_prev),XtoScrX(x),YtoScrY(y),color);	// 前回の座標から今回の座標まで線を引く
	x_prev = x;	// 次回用に格納
	y_prev = y;	// 次回用に格納
}
// 関数オーバーロードここまで

void CuiPlot::PlotVariables(volatile double x, volatile double* var_values){
	// 複数の変数値を逐次プロットする関数
	if(IsVisible == false)return;	// 不可視設定なら何もしない
	
	// 最初の1回はプロットしない
	if(IsFirstTime == true){
		IsFirstTime = false;
		for(unsigned int i=0;i<NumOfVars;i++){
			var_values_prev[i] = var_values[i];	// 次回用に格納
		}
		x_prev = x;	// 次回用に格納
		return;
	}
	
	for(unsigned int i=0;i<NumOfVars;i++){
		// 前回の座標から今回の座標まで線を引く
		pFG->DrawLine(XtoScrX(x_prev),YtoScrY(var_values_prev[i]),XtoScrX(x),YtoScrY(var_values[i]),LineColors[i]);
		var_values_prev[i] = var_values[i];	// 次回用に格納
	}
	x_prev = x;	// 次回用に格納
}

void CuiPlot::EndPlot(){
	// 逐次プロット終了
	IsFirstTime = true;	// 最初の1回判定フラグを初期値に戻す
}

void CuiPlot::ClearPlots(){
	// プロットの消去
	if(IsVisible == false)return;	// 不可視設定なら何もしない
	pFG->DrawRectFill(XtoScrX(Xmin), YtoScrY(Ymax), ScrWidth, ScrHeight, BackColor);
	DrawGrids();
}

void CuiPlot::SetXrange(double xmin, double xmax){
	// X軸範囲の設定
	Xmin=xmin;
	Xmax=xmax;
	XgridCalc();	// グリッド座標再計算
}

void CuiPlot::SetYrange(double ymin, double ymax){
	// Y軸範囲の設定
	Ymin=ymin;
	Ymax=ymax;
	YgridCalc();	// グリッド座標再計算
}

void CuiPlot::SetXgridNum(unsigned int N){
	// X軸グリッド分割数の設定
	XgridNum=N;
	XgridCalc();	// グリッド座標再計算
}

void CuiPlot::SetYgridNum(unsigned int N){
	// Y軸グリッド分割数の設定
	YgridNum=N;
	YgridCalc();// グリッド座標再計算
}


void CuiPlot::SetXLabel(std::string str){
	// X軸ラベルの設定
	XLabel=str;
}

void CuiPlot::SetYLabel(std::string str){
	// Y軸ラベルの設定
	YLabel=str;
}

void CuiPlot::SetXform(std::string format){
	// X軸書式の設定
	Xform=format;
}

void CuiPlot::SetYform(std::string format){
	// Y軸書式の設定
	Yform=format;
}

void CuiPlot::SetVisible(bool flag){
	// グラフ描画の有効/無効設定
	IsVisible = flag;
}

