//! @file CuiPlot.hh
//! @brief CuiPlotクラス
//!
//! CUI(ランレベル3)の画面上にグラフの描画を行います。ただし，FrameGraphicsクラス が必要。
//!
//! @date 2018/11/30
//! @author Yuki YOKOKURA
//
// Copyright (C) 2011-2018 Yuki YOKOKURA
// This program is free software;
// you can redistribute it and/or modify it under the terms of the FreeBSD License.
// For details, see the License.txt file.

#ifndef CUIPLOT
#define CUIPLOT

#include <string>
#include "FrameGraphics.hh"

namespace ARCS {
	class CuiPlot {
		public:
			CuiPlot(FrameGraphics& FG, int x, int y, int w, int h);	//!< コンストラクタ
			CuiPlot();												//!< 空コンストラクタ
			CuiPlot(CuiPlot&& r);									//!< ムーブコンストラクタ
			~CuiPlot();												//!< デストラクタ
			
			void SetAxisColor(FrameGraphics::FGcolors color);	// 主軸の色設定
			void SetGridColor(FrameGraphics::FGcolors color);	// グリッドの色設定
			void SetBackColor(FrameGraphics::FGcolors color);	// 背景色の設定
			void SetTextColor(FrameGraphics::FGcolors color);	// 文字色の設定
			void SetXrange(double xmin, double xmax);			// X軸範囲の設定
			void SetYrange(double ymin, double ymax);			// Y軸範囲の設定
			void SetXgridNum(unsigned int N);					// X軸グリッド分割数の設定
			void SetYgridNum(unsigned int N);					// Y軸グリッド分割数の設定
			void SetXLabel(std::string str);					// X軸ラベルの設定
			void SetYLabel(std::string str);					// Y軸ラベルの設定
			void SetXform(std::string format);					// X軸書式の設定
			void SetYform(std::string format);					// Y軸書式の設定
			void DrawAxis();									// 軸の描画
			void DrawLegend();									// 凡例の描画
			void PlotData(double* x, double* y, unsigned int N, FrameGraphics::FGcolors color);	// 1つの変数値を一括プロットする関数
			void PlotData(double x, double y, FrameGraphics::FGcolors color);					// 1つの変数値を逐次プロットする関数
			void SetVariables(const unsigned int var_num, const std::string* var_names, const FrameGraphics::FGcolors* var_colors);	// プロットする変数の設定
			void PlotVariables(volatile double x, volatile double* var_values);	// 複数の変数値を逐次プロットする関数
			void EndPlot();										// 逐次プロット終了
			void ClearPlots();									// プロットの消去
			void SetVisible(bool flag);							// グラフ描画の有効/無効設定
			
		private:
			CuiPlot(const CuiPlot&) = delete;					//!< コピーコンストラクタ使用禁止
			const CuiPlot& operator=(const CuiPlot&) = delete;	//!< 代入演算子使用禁止
			static const int LABEL_VERTICAL_ALIGN = 4;	// [px] ラベルの縦位置補正用
			static const int LEGEND_LEFT = 120;			// [px] 凡例の横位置
			static const int LEGEND_INTERVAL = 75;		// [px] 凡例の間隔
			static const int LEGEND_LINE_LENGTH = 18;	// [px] 凡例の線の長さ
			static const unsigned int MAX_VARIABLE_NUM = 8;	// 表示可能な変数の最大数
			
			FrameGraphics* pFG;	// フレームグラフィックスクラスへのポインタ
			FrameGraphics::FGcolors AxisColor;	// 主軸の色
			FrameGraphics::FGcolors GridColor;	// グリッドの色
			FrameGraphics::FGcolors BackColor;	// 背景の色
			FrameGraphics::FGcolors TextColor;	// 文字の色
			int ScrLeft;			// 横位置
			int ScrTop;				// 縦位置
			int ScrWidth;			// 横幅
			int ScrHeight;			// 高さ
			double Xmax;			// X最大値
			double Xmin;			// X最小値
			double Ymax;			// Y最大値
			double Ymin;			// Y最小値
			unsigned int XgridNum;	// X軸グリッドの分割数
			double* Xgrid;			// X軸グリッドX座標
			unsigned int YgridNum;	// Y軸グリッドの分割数
			double* Ygrid;			// Y軸グリッドY座標
			int ScaleLen;			// 目盛りの長さ
			int XLabelMargin;		// Xラベルマージン
			int YLabelMargin;		// Yラベルマージン
			int MarginWidth;		// 横幅余白
			int MarginHeight;		// 高さ余白
			std::string XLabel;		// X軸ラベル
			std::string YLabel;		// Y軸ラベル
			std::string Xform;		// X軸書式
			std::string Yform;		// Y軸書式
			unsigned int NumOfVars;	// プロットする変数の数
			std::string* LegendNames;			// 凡例に表示する変数名
			FrameGraphics::FGcolors* LineColors;// 線の色
			double x_prev;			// 逐次プロット用 前回のx
			double y_prev;			// 逐次プロット用 前回のy
			double* var_values_prev;// 複数逐次プロット用 前回の変数値
			bool IsFirstTime;		// 逐次プロット用 最初の1回判定フラグ
			bool IsVisible;			// グラフ描画の有効/無効設定
			
			void XgridCalc(void);	// X軸グリッドの計算
			void YgridCalc(void);	// Y軸グリッドの計算
			int XtoScrX(double X) const;	// X実数値からX画面座標へ変換する関数
			int YtoScrY(double Y) const;	// Y実数値からY画面座標へ変換する関数
			void DrawGrids();		// グリッドの描画
			void DrawLabels();		// ラベルの描画
	};
}

#endif

