//! @file FrameGraphics.cc
//! @brief フレームバッファグラフィックスクラス
//!
//! 画面上に図等の描画を行います。
//! ただし，/dev/fb* が必要で，ディスプレイの色深度は16bitか32bitのみ対応。gccとフレームバッファさえあれば動く。
//!
//! @date 2019/01/17
//! @author Yuki YOKOKURA
//
// Copyright (C) 2011-2019 Yuki YOKOKURA
// This program is free software;
// you can redistribute it and/or modify it under the terms of the FreeBSD License.
// For details, see the License.txt file.

#include <unistd.h>
#include <stdio.h>
#include <fcntl.h>
#include <linux/fs.h>
#include <sys/mman.h>
#include <sys/ioctl.h>
#include <string.h>
#include <stdint.h>
#include <errno.h>
#include <math.h>
#include "FrameGraphics.hh"
#include "ARCSassert.hh"
#include "ARCSeventlog.hh"

using namespace ARCS;

FrameGraphics::FrameGraphics(const char* device)
	// コンストラクタ
	// フレームバッファを開いてメモリにマップする
	: isEnabled(true), FBfd(0), FBptr16(0), FBptr32(0), finfo(), vinfo(), width(0), height(0), depth(0), size(0), length(0),
	  xofst(0), yofst(0), xlen(0), bppx(0), IsFontDataLocked(false)
{
	// フレームバッファデバイスを開く
	PassedLog();
	FBfd = open(device, O_RDWR, 0);
	arcs_assert(FBfd != 0 && "[ERROR] FrameGraphics : could not open the frame buffer device.");
	
	// 固定画面情報の取得
	ioctl(FBfd, FBIOGET_FSCREENINFO, &finfo);
	xlen = finfo.line_length;
	
	// 変動画面情報の取得
	ioctl(FBfd, FBIOGET_VSCREENINFO, &vinfo);
	width = vinfo.xres;
	height = vinfo.yres;
	depth = vinfo.bits_per_pixel;
	xofst = vinfo.xoffset;
	yofst = vinfo.yoffset;
	bppx = vinfo.bits_per_pixel;
	
	// 色深度の判定
	if(depth != 16 && depth != 32){
		isEnabled = false;	// 未対応色深度のときは無効にする
		EventLog("FrameGraphics is disabled.");
		EventLogVar(depth);
	}
	
	// フレームバッファの長さを計算
	length=(long)width*(long)height;
	EventLogVar(length);
	
	// 1画面データの大きさを計算 [bytes]
	size = length*(long)depth/8;
	EventLogVar(size);
	
	// フレームバッファをメモリにマップ(こんなことしないでchar*でやればいいじゃんと思ったそこのあなた，正解！)
	if(isEnabled == true){
		if(depth==16){
			// 色深度が16bitのとき
			FBptr16 = (int16_t*)mmap(NULL, size, PROT_READ|PROT_WRITE, MAP_SHARED, FBfd, 0);
			arcs_assert(FBptr16 != MAP_FAILED && "[ERROR] FrameGraphics : could not map using mmap().");
		}
		if(depth==32){
			// 色深度が32bitのとき
			FBptr32 = (int32_t*)mmap(NULL, size, PROT_READ|PROT_WRITE, MAP_SHARED, FBfd, 0);
			arcs_assert(FBptr32 != MAP_FAILED && "[ERROR] FrameGraphics : could not map using mmap().");
		}
	}
	
	// フォントデータの準備
	PrepareFontData(WHITE,BLACK);	// 初期値は白文字に黒背景
	PassedLog();
}

//! @brief ムーブコンストラクタ
FrameGraphics::FrameGraphics(FrameGraphics&& r)
	: isEnabled(r.isEnabled), FBfd(r.FBfd), FBptr16(r.FBptr16), FBptr32(r.FBptr32),
	  finfo(r.finfo), vinfo(r.vinfo), width(r.width), height(r.height), depth(r.depth), size(r.size), length(r.length),
	  xofst(r.xofst), yofst(r.yofst), xlen(r.xlen), bppx(r.bppx), IsFontDataLocked(r.IsFontDataLocked)
{
	PassedLog();
}

FrameGraphics::~FrameGraphics(void){
	// デストラクタ
	PassedLog();
	// フレームバッファを閉じる
	if(depth==16)munmap(FBptr16, size);
	if(depth==32)munmap(FBptr32, size);
	close(FBfd);
	PassedLog();
}

void FrameGraphics::ShowParam(void){
	// 画面情報の表示
	printf("WIDTH  = %d px \n",width);		// 画面の幅
	printf("HEIGHT = %d px \n",height);		// 画面の高さ
	printf("DEPTH  = %d bits \n",depth);	// 色深度をビット数で表示
	printf("SIZE   = %ld bytes \n",size);	// 画面サイズをバイト数で表示
	printf("LENGTH = %ld \n",length);		// 全要素数
	printf("XLENGTH= %d \n",xlen);			// x方向要素数
	printf("XOFFSET= %d px \n",xofst);		// x方向オフセット
	printf("YOFFSET= %d px \n",yofst);		// y方向オフセット
	printf("BIT/PX = %d bits/px \n",bppx);	// ピクセル当りのビット数
}

void FrameGraphics::DrawPoint(int x, int y, FGcolors color){
	// フレームバッファに点(x,y)を描画する
	// color：色
	long i=(long)width*(long)y+(long)x;		// 座標からフレームバッファの要素番号を計算
	
	if(i<0)i=0;				// 要素番号リミッタ
	if(length<i)i=length-1;	// 要素番号リミッタ
	
	if(depth == 16 and isEnabled == true)FBptr16[i]=ColorSet16[color];	// フレームバッファへ書き込み
	if(depth == 32 and isEnabled == true)FBptr32[i]=ColorSet32[color];	// フレームバッファへ書き込み
}

void FrameGraphics::DrawPoint(int x, int y, double r, double g, double b){
	// フレームバッファに点(x,y)を描画する
	// color：色
	long i=(long)width*(long)y+(long)x;		// 座標からフレームバッファの要素番号を計算
	
	if(i<0)i=0;				// 要素番号リミッタ
	if(length<i)i=length-1;	// 要素番号リミッタ
	
	if(depth == 16 and isEnabled == true)FBptr16[i]=RGBcolorToData16(r,g,b);	// フレームバッファへ書き込み
	if(depth == 32 and isEnabled == true)FBptr32[i]=RGBcolorToData32(r,g,b);	// フレームバッファへ書き込み
}

void FrameGraphics::DrawLine(int x1, int y1, int x2, int y2, FGcolors color){
	// フレームバッファに直線(x1,y1)ー(x2,y2)を引く関数
	
	// 垂直の線のとき(ブレゼンハムのアルゴリズムでは描けないので，別に実装)
	if(x1 == x2){
		int ysta, yend;
		if(y1 < y2){
			// y1→y2で増加する場合
			ysta = y1;
			yend = y2;
		}else{
			// y2→y1で増加する場合
			ysta = y2;
			yend = y1;
		}
		for(int y=ysta;y<=yend;y++)DrawPoint(x1,y,color);	// 直線に沿って点を描画
		return;
	}
	
	// 水平の線のとき(ブレゼンハムのアルゴリズムでも描けるが，高速化のために別に実装)
	if( y1 == y2 ){
		int xsta, xend;
		if(x1 < x2){
			// x1→x2で増加する場合
			xsta = x1;
			xend = x2;
		}else{
			// x2→x1で増加する場合
			xsta = x2;
			xend = x1;
		}
		for(int x=xsta;x<=xend;x++)DrawPoint(x,y1,color);	// 直線に沿って点を描画
		return;
	}
	
	// 以下はブレゼンハムのアルゴリズム
	bool IsHighGradient = std::abs(x2 - x1) < std::abs(y2 - y1);	// 直線の傾きが45度を境に急勾配かどうかのフラグ
	if(IsHighGradient == true){		// 45度を境に急勾配のとき
		std::swap(x1, y1);	// x1とy1を入れ替え
		std::swap(x2, y2);	// x2とy2を入れ替え
	}
	if(x2 < x1){	// 始点の方が終点よりも右のとき
		std::swap(x1, x2);
		std::swap(y1, y2);
	}
	int Dx = x2 - x1;
	int Dy = std::abs(y2 - y1);
	int Error = Dx/2;
	int ystep;
	if(y1 < y2){
		ystep = 1;
	}else{
		ystep = -1;
	}
	int y = y1;
	for(int x=x1;x<=x2;x++){
		if(IsHighGradient == true){
			DrawPoint(y,x,color);
		}else{
			DrawPoint(x,y,color);
		}
		Error -= Dy;
		if(Error < 0){
			y += ystep;
			Error += Dx;
		}
	}
}

void FrameGraphics::DrawLine(int x1, int y1, int x2, int y2, double r, double g, double b){
	// フレームバッファに直線(x1,y1)ー(x2,y2)を引く関数
	
	// 垂直の線のとき(ブレゼンハムのアルゴリズムでは描けないので，別に実装)
	if(x1 == x2){
		int ysta, yend;
		if(y1 < y2){
			// y1→y2で増加する場合
			ysta = y1;
			yend = y2;
		}else{
			// y2→y1で増加する場合
			ysta = y2;
			yend = y1;
		}
		for(int y=ysta;y<=yend;y++)DrawPoint(x1,y,r,g,b);	// 直線に沿って点を描画
		return;
	}
	
	// 水平の線のとき(ブレゼンハムのアルゴリズムでも描けるが，高速化のために別に実装)
	if( y1 == y2 ){
		int xsta, xend;
		if(x1 < x2){
			// x1→x2で増加する場合
			xsta = x1;
			xend = x2;
		}else{
			// x2→x1で増加する場合
			xsta = x2;
			xend = x1;
		}
		for(int x=xsta;x<=xend;x++)DrawPoint(x,y1,r,g,b);	// 直線に沿って点を描画
		return;
	}
	
	// 以下はブレゼンハムのアルゴリズム
	bool IsHighGradient = std::abs(x2 - x1) < std::abs(y2 - y1);	// 直線の傾きが45度を境に急勾配かどうかのフラグ
	if(IsHighGradient == true){		// 45度を境に急勾配のとき
		std::swap(x1, y1);	// x1とy1を入れ替え
		std::swap(x2, y2);	// x2とy2を入れ替え
	}
	if(x2 < x1){	// 始点の方が終点よりも右のとき
		std::swap(x1, x2);
		std::swap(y1, y2);
	}
	int Dx = x2 - x1;
	int Dy = std::abs(y2 - y1);
	int Error = Dx/2;
	int ystep;
	if(y1 < y2){
		ystep = 1;
	}else{
		ystep = -1;
	}
	int y = y1;
	for(int x=x1;x<=x2;x++){
		if(IsHighGradient == true){
			DrawPoint(y,x,r,g,b);
		}else{
			DrawPoint(x,y,r,g,b);
		}
		Error -= Dy;
		if(Error < 0){
			y += ystep;
			Error += Dx;
		}
	}
}

void FrameGraphics::DrawRect(int x, int y, int w, int h, FGcolors color){
	// 長方形の描画をする（色は塗らない）
	// x, y；長方形左上の座標，w；長方形の幅，h；長方形の高さ，color；色
	DrawLine(x  , y  , x+w, y  , color);
	DrawLine(x+w, y  , x+w, y+h, color);
	DrawLine(x+w, y+h, x  , y+h, color);
	DrawLine(x  , y+h, x  , y  , color);
}
void FrameGraphics::DrawRect(int x, int y, int w, int h, double r, double g, double b){
	// 長方形の描画をする（色は塗らない）
	// x, y；長方形左上の座標，w；長方形の幅，h；長方形の高さ，r,g,b；色
	DrawLine(x  , y  , x+w, y  , r, g, b);
	DrawLine(x+w, y  , x+w, y+h, r, g, b);
	DrawLine(x+w, y+h, x  , y+h, r, g, b);
	DrawLine(x  , y+h, x  , y  , r, g, b);
}

void FrameGraphics::DrawRectFill(int x, int y, int w, int h, FGcolors color){
	// 長方形の範囲内に色を塗る
	// x, y；長方形左上の座標，w；長方形の幅，h；長方形の高さ，color；色
	int i,j;
	
	for(j=y;j<=y+h;j++){
		for(i=x;i<=x+w;i++){
			DrawPoint(i,j,color);
		}
	}
}
void FrameGraphics::DrawRectFill(int x, int y, int w, int h, double r, double g, double b){
	// 長方形の範囲内に色を塗る
	// x, y；長方形左上の座標，w；長方形の幅，h；長方形の高さ，r,g,b；色
	for(int j=y;j<=y+h;j++){
		for(int i=x;i<=x+w;i++){
			DrawPoint(i,j,r,g,b);
		}
	}
}

void FrameGraphics::DrawCircle(int cx, int cy, int radius, unsigned int N, FGcolors color){
	// 円の描画をする
	// cx,cy：円の中心座標，radius：半径，N：円の分割数
	const double TwoPIdivN = 2.0*M_PI/(double)N;
	int x_prev = (double)radius + cx;
	int y_prev = cy;
	for(unsigned int i=0;i<=N;i++){
		double theta = TwoPIdivN*(double)i;
		int x = (double)radius*cos(theta) + cx;
		int y = (double)radius*sin(theta) + cy;
		DrawLine(x_prev,y_prev, x,y, color);
		x_prev = x;
		y_prev = y;
	}
}
void FrameGraphics::DrawCircle(int cx, int cy, int radius, unsigned int N, double r, double g, double b){
	// 円の描画をする
	// cx,cy：円の中心座標，radius：半径，N：円の分割数
	const double TwoPIdivN = 2.0*M_PI/(double)N;
	int x_prev = (double)radius + cx;
	int y_prev = cy;
	for(unsigned int i=0;i<=N;i++){
		double theta = TwoPIdivN*(double)i;
		int x = (double)radius*cos(theta) + cx;
		int y = (double)radius*sin(theta) + cy;
		DrawLine(x_prev,y_prev, x,y, r,g,b);
		x_prev = x;
		y_prev = y;
	}
}

int16_t FrameGraphics::RGBcolorToData16(double Red, double Green, double Blue){
	// 0～1の浮動小数点数値を赤緑青色の16bit輝度データに変える
	int16_t r,g,b;
	r = DoubleToRedIntensity16(Red);	// 赤を変換
	g = DoubleToGreenIntensity16(Green);// 緑を変換
	b = DoubleToBlueIntensity16(Blue);	// 青を変換
	return r | g | b;					// ORをとって返す
}
int16_t FrameGraphics::DoubleToIntensity16(double u){
	// 0～1の浮動小数点数値を白色の16bit輝度データに変える
	if(1 < u)u = 1;
	if(u < 0)u = 0;
	return (int16_t)((double)0xFFFF*u);
}
int16_t FrameGraphics::DoubleToRedIntensity16(double u){
	// 0～1の浮動小数点数値を赤色の16bit輝度データに変える
	if(1 < u)u = 1;
	if(u < 0)u = 0;
	return ((int16_t)((double)0x001F*u)) << 11;
}
int16_t FrameGraphics::DoubleToGreenIntensity16(double u){
	// 0～1の浮動小数点数値を緑色の16bit輝度データに変える
	if(1 < u)u = 1;
	if(u < 0)u = 0;
	return ((int16_t)((double)0x003F*u)) << 5;
}
int16_t FrameGraphics::DoubleToBlueIntensity16(double u){
	// 0～1の浮動小数点数値を青色の16bit輝度データに変える
	if(1 < u)u = 1;
	if(u < 0)u = 0;
	return ((int16_t)((double)0x001F*u));
}

int32_t FrameGraphics::RGBcolorToData32(double Red, double Green, double Blue){
	// 0～1の浮動小数点数値を赤緑青色の32bit輝度データに変える
	int32_t r,g,b;
	r = DoubleToRedIntensity32(Red);	// 赤を変換
	g = DoubleToGreenIntensity32(Green);// 緑を変換
	b = DoubleToBlueIntensity32(Blue);	// 青を変換
	return r | g | b;					// ORをとって返す
}
int32_t FrameGraphics::DoubleToIntensity32(double u){
	// 0～1の浮動小数点数値を白色の32bit輝度データに変える
	if(1 < u)u = 1;
	if(u < 0)u = 0;
	return (int32_t)((double)0x00FFFFFF*u);
}
int32_t FrameGraphics::DoubleToRedIntensity32(double u){
	// 0～1の浮動小数点数値を赤色の32bit輝度データに変える
	if(1 < u)u = 1;
	if(u < 0)u = 0;
	return ((int32_t)((double)0x000000FF*u)) << 16;
}
int32_t FrameGraphics::DoubleToGreenIntensity32(double u){
	// 0～1の浮動小数点数値を緑色の32bit輝度データに変える
	if(1 < u)u = 1;
	if(u < 0)u = 0;
	return ((int32_t)((double)0x000000FF*u)) << 8;
}
int32_t FrameGraphics::DoubleToBlueIntensity32(double u){
	// 0～1の浮動小数点数値を青色の32bit輝度データに変える
	if(1 < u)u = 1;
	if(u < 0)u = 0;
	return ((int32_t)((double)0x000000FF*u));
}

void FrameGraphics::ClearScreen(void){
	// 全画面を消去
	if(depth==16)memset(FBptr16,0,length-1);
	if(depth==32)memset(FBptr32,0,length-1);
}

void FrameGraphics::ClearRect(int x, int y, int w, int h){
	// 長方形の範囲内を消去
	// x, y；長方形左上の座標，w；長方形の幅，h；長方形の高さ
	DrawRectFill(x,y,w,h,BLACK);
}

int FrameGraphics::GetWidth(void) const {
	// 画面の幅の取得
	return width;
}

int FrameGraphics::GetHeight(void) const {
	// 画面の高さの取得
	return height;
}

int FrameGraphics::GetDepth(void) const {
	// 画面の色深度の取得
	return depth;
}

void FrameGraphics::PrintValue(int x, int y, FGalign align, std::string format, double val){
	// 指定した書式で文字列を描画する x：[px] 横位置，y：[px] 縦位置，align：揃え位置，format：書式指定子，val：所望の数値
	// formatの書式指定子は printf関数 の場合の書き方と同等
	char str[TEXT_MAXLEN] = {'\0'};
	sprintf(str, format.c_str(), val);
	PrintText(x,y,align,str);
}

void FrameGraphics::PrintText(int x, int y, FGalign align, std::string text){
	// 文字列を描画する x：[px]横位置，y：[px] 縦位置，align：揃え位置，text：所望の文字列
	
	// 文字列の揃え指定に従って座標にオフセットを持たせる
	switch(align){
		case ALIGN_LEFT:
			// 左揃えのときはなにもしない
			break;
		case ALIGN_CENTER:
			// 中央揃えのときは横位置から文字列の長さの半分を引く
			x = x - text.length()*(FONT_WIDTH + TEXT_INTERVAL)/2;
			break;
		case ALIGN_RIGHT:
			// 右揃えのときは横位置から文字列の長さを引く
			x = x - text.length()*(FONT_WIDTH + TEXT_INTERVAL);
			break;
	}
	
	// 1文字ずつ描画
	for(size_t i=0;i<text.length();i++){
		WriteFont(x + i*(FONT_WIDTH + TEXT_INTERVAL), y, (unsigned int)text.at(i));
	}
}

void FrameGraphics::WriteFont(int x, int y, unsigned int ascii){
	// 1文字分のフォントデータをフレームバッファに書き込む
	// 注意！:フォントデータが準備中のときはなにもしない
	// x：[px] 横位置，y：[px] 縦位置，ascii：アスキーコード
	unsigned int j;
	unsigned int FontWidthInByte;		// フォントデータ1ラインのバイト数
	long i=(long)width*(long)y+(long)x;	// 座標からフレームバッファの要素番号を計算
	
	// フォントデータが準備中のときは何もせずに抜ける
	if(IsFontDataLocked == true)return;
	
	// アスキーコードが範囲外のときは範囲外用のフォントを使用
	if(ascii < FONT_FST_ASCII || FONT_END_ASCII < ascii)ascii = FONT_END_ASCII + 1;
	// アスキーコードからフォントデータの要素番号へ変換
	ascii = ascii - FONT_FST_ASCII;
	
	// 16bit色深度用
	if(depth == 16){
		FontWidthInByte = sizeof(int16_t)*FONT_WIDTH;	// フォントデータ1ラインのバイト数
		for(j=0;j<FONT_HEIGHT;j++){
			// フォントデータを1ラインずつフレームバッファへ書き込む
			memcpy(FBptr16 + i + width*j, FontPrepared16[ascii][j], FontWidthInByte);
		}
	}
	
	// 32bit色深度用
	if(depth == 32){
		FontWidthInByte = sizeof(int32_t)*FONT_WIDTH;	// フォントデータ1ラインのバイト数
		for(j=0;j<FONT_HEIGHT;j++){
			// フォントデータを1ラインずつフレームバッファへ書き込む
			memcpy(FBptr32 + i + width*j, FontPrepared32[ascii][j], FontWidthInByte);
		}
	}
}

void FrameGraphics::PrepareFontData(FGcolors fore_color, FGcolors back_color){
	// 設定した色のフォントデータを準備
	// この関数は結構遅いはずなので注意。アホっぽいので設計から見直す必要あり。
	unsigned int i,j,k;
	
	IsFontDataLocked = true;	// フォントデータスピンロック
	
	// フォントデータの中身を書き換え (16bit色深度用)
	if(depth == 16){
		for(i=0;i<FONT_NUM;i++){
			for(j=0;j<FONT_HEIGHT;j++){
				for(k=0;k<FONT_WIDTH;k++){
					if(FONT_DATA[i][j][k] == 1){
						FontPrepared16[i][j][k] = ColorSet16[fore_color];	// 前景色
					}else{
						FontPrepared16[i][j][k] = ColorSet16[back_color];	// 背景色
					}
				}
			}
		}
	}
	
	// フォントデータの中身を書き換え (32bit色深度用)
	if(depth == 32){
		for(i=0;i<FONT_NUM;i++){
			for(j=0;j<FONT_HEIGHT;j++){
				for(k=0;k<FONT_WIDTH;k++){
					if(FONT_DATA[i][j][k] == 1){
						FontPrepared32[i][j][k] = ColorSet32[fore_color];	// 前景色
					}else{
						FontPrepared32[i][j][k] = ColorSet32[back_color];	// 背景色
					}
				}
			}
		}
	}
	
	IsFontDataLocked = false;	// フォントデータスピンロック解除
}

void FrameGraphics::PrintAllFontData(){
	// 収録されているすべてのフォントデータを描画するテスト用関数
	for(unsigned int i=FONT_FST_ASCII;i<=FONT_END_ASCII;i++){
		WriteFont((i-FONT_FST_ASCII)*(FONT_WIDTH + TEXT_INTERVAL),0,i);
	}
}

// 色の定義 (色深度16bitモード用) 注意!!：enum FGcolors の定義と相違が無いようにせよ！
// ビットパターン MSB RRRRRGGGGGGBBBBB LSB (赤：ビット15～11 緑：ビット10～5 青：ビット4～0)
// 実は赤青に比べ緑の分解能が1bit分だけ多い(人間の網膜の特性による)
// 以下は基本色の定義  この他にも勝手に好きな色を作れる
const int16_t FrameGraphics::ColorSet16[NUM_COLOR_SET] = {
	(int16_t)0xF800,	// RED
	(int16_t)0x07E0,	// GREEN
	(int16_t)0x001F,	// BLUE
	(int16_t)0x07FF,	// CYAN
	(int16_t)0xF81F,	// MAGENTA
	(int16_t)0xFFE0,	// YELLOW
	(int16_t)0xF800,	// ORANGE
	(int16_t)0xFFFF,	// WHITE
	(int16_t)0xB4F6,	// GRAY75
	(int16_t)0x7BEF,	// GRAY50
	(int16_t)0x39E7,	// GRAY25
	(int16_t)0x0000		// BLACK
};


// 色の定義 (32bit色深度用) 注意!!：enum FGcolors の定義と相違が無いようにせよ！
// ビットパターン MSB AAAAAAAA RRRRRRRR GGGGGGGG BBBBBBBB LSB (α：ビット31～25 赤：ビット24～16 緑：ビット15～8 青：ビット7～0)
// αは透過レベル
// 以下は基本色の定義  この他にも勝手に好きな色を作れる
const int32_t FrameGraphics::ColorSet32[NUM_COLOR_SET] = {
	0x00FF0000,	// RED
	0x0000FF00,	// GREEN
	0x000000FF,	// BLUE
	0x0000FFFF,	// CYAN
	0x00FF00FF,	// MAGENTA
	0x00FFFF00,	// YELLOW
	0x00FF8000,	// ORANGE
	0x00FFFFFF,	// WHITE
	0x00C0C0C0,	// GRAY75
	0x00808080,	// GRAY50
	0x00404040,	// GRAY25
	0x00000000	// BLACK
};

// フォントデータ
// (フォントが気に食わない人は各自で好きなように変えてネ)
const int FrameGraphics::FONT_DATA[FONT_NUM][FONT_HEIGHT][FONT_WIDTH] = {
	{	// " " ASCII CODE = 32
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0}
	},
	{	// "!" ASCII CODE = 33
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0}
	},
	{	// """ ASCII CODE = 34
		{0, 1, 0, 1, 0},
		{0, 1, 0, 1, 0},
		{0, 1, 0, 1, 0},
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0}
	},
	{	// "#" ASCII CODE = 35
		{0, 1, 0, 1, 0},
		{0, 1, 0, 1, 0},
		{1, 1, 1, 1, 1},
		{0, 1, 0, 1, 0},
		{0, 1, 0, 1, 0},
		{0, 1, 0, 1, 0},
		{1, 1, 1, 1, 1},
		{0, 1, 0, 1, 0},
		{0, 1, 0, 1, 0}
	},
	{	// "$" ASCII CODE = 36
		{0, 0, 1, 0, 0},
		{0, 1, 1, 1, 0},
		{1, 0, 1, 0, 1},
		{1, 0, 1, 0, 0},
		{0, 1, 1, 1, 0},
		{0, 0, 1, 0, 1},
		{1, 0, 1, 0, 1},
		{0, 1, 1, 1, 0},
		{0, 0, 1, 0, 0}
	},
	{	// "%" ASCII CODE = 37
		{0, 1, 0, 0, 1},
		{1, 0, 1, 0, 1},
		{0, 1, 0, 1, 0},
		{0, 0, 0, 1, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 1, 0, 1, 0},
		{0, 1, 1, 0, 1},
		{1, 0, 0, 1, 0}
	},
	{	// "&" ASCII CODE = 38
		{0, 0, 1, 0, 0},
		{0, 1, 0, 1, 0},
		{0, 1, 0, 1, 0},
		{0, 0, 1, 0, 0},
		{0, 1, 1, 0, 0},
		{1, 0, 1, 0, 1},
		{1, 0, 0, 1, 1},
		{1, 0, 0, 0, 1},
		{0, 1, 1, 1, 0}
	},
	{	// "'" ASCII CODE = 39
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0}
	},
	{	// "(" ASCII CODE = 40
		{0, 0, 0, 1, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 1, 0, 0, 0},
		{0, 1, 0, 0, 0},
		{0, 1, 0, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 0, 1, 0}
	},
	{	// ")" ASCII CODE = 41
		{0, 1, 0, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 0, 1, 0},
		{0, 0, 0, 1, 0},
		{0, 0, 0, 1, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 1, 0, 0, 0}
	},
	{	// "*" ASCII CODE = 42
		{0, 0, 0, 0, 0},
		{0, 0, 1, 0, 0},
		{1, 0, 1, 0, 1},
		{0, 1, 1, 1, 0},
		{1, 1, 1, 1, 1},
		{0, 1, 1, 1, 0},
		{1, 0, 1, 0, 1},
		{0, 0, 1, 0, 0},
		{0, 0, 0, 0, 0}
	},
	{	// "+" ASCII CODE = 43
		{0, 0, 0, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{1, 1, 1, 1, 1},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 0, 0, 0}
	},
	{	// "," ASCII CODE = 44
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 1, 0, 0, 0}
	},
	{	// "-" ASCII CODE = 45
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0},
		{1, 1, 1, 1, 1},
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0}
	},
	{	// "." ASCII CODE = 46
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0}
	},
	{	// "/" ASCII CODE = 47
		{0, 0, 0, 0, 1},
		{0, 0, 0, 0, 1},
		{0, 0, 0, 1, 0},
		{0, 0, 0, 1, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 1, 0, 0, 0},
		{0, 1, 0, 0, 0},
		{1, 0, 0, 0, 0}
	},
	{	// "0" ASCII CODE = 48
		{0, 1, 1, 1, 0},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 1, 1},
		{1, 0, 1, 0, 1},
		{1, 1, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{0, 1, 1, 1, 0}
	},
	{	// "1" ASCII CODE = 49
		{0, 0, 1, 0, 0},
		{0, 1, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 1, 1, 1, 0}
	},
	{	// "2" ASCII CODE = 50
		{0, 1, 1, 1, 0},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{0, 0, 0, 0, 1},
		{0, 0, 0, 1, 0},
		{0, 0, 1, 0, 0},
		{0, 1, 0, 0, 0},
		{1, 0, 0, 0, 1},
		{1, 1, 1, 1, 1}
	},
	{	// "3" ASCII CODE = 51
		{0, 1, 1, 1, 0},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{0, 0, 0, 0, 1},
		{0, 0, 1, 1, 0},
		{0, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{0, 1, 1, 1, 0}
	},
	{	// "4" ASCII CODE = 52
		{0, 0, 0, 1, 0},
		{0, 0, 1, 1, 0},
		{0, 0, 1, 1, 0},
		{0, 1, 0, 1, 0},
		{0, 1, 0, 1, 0},
		{1, 0, 0, 1, 0},
		{1, 1, 1, 1, 1},
		{0, 0, 0, 1, 0},
		{0, 0, 0, 1, 0}
	},
	{	// "5" ASCII CODE = 53
		{1, 1, 1, 1, 1},
		{1, 0, 0, 0, 0},
		{1, 0, 0, 0, 0},
		{1, 1, 1, 1, 0},
		{0, 0, 0, 0, 1},
		{0, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{0, 1, 1, 1, 0}
	},
	{	// "6" ASCII CODE = 54
		{0, 1, 1, 1, 0},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 0},
		{1, 1, 1, 1, 0},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{0, 1, 1, 1, 0}
	},
	{	// "7" ASCII CODE = 55
		{1, 1, 1, 1, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{0, 0, 0, 1, 0},
		{0, 0, 0, 1, 0},
		{0, 0, 0, 1, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0}
	},
	{	// "8" ASCII CODE = 56
		{0, 1, 1, 1, 0},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{0, 1, 1, 1, 0},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{0, 1, 1, 1, 0}
	},
	{	// "9" ASCII CODE = 57
		{0, 1, 1, 1, 0},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{0, 1, 1, 1, 1},
		{0, 0, 0, 0, 1},
		{0, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{0, 1, 1, 1, 0}
	},
	{	// ":" ASCII CODE = 58
		{0, 0, 0, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 0, 0, 0}
	},
	{	// ";" ASCII CODE = 59
		{0, 0, 0, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 1, 0, 0, 0}
	},
	{	// "<" ASCII CODE = 60
		{0, 0, 0, 0, 1},
		{0, 0, 0, 1, 0},
		{0, 0, 1, 0, 0},
		{0, 1, 0, 0, 0},
		{1, 0, 0, 0, 0},
		{0, 1, 0, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 0, 1, 0},
		{0, 0, 0, 0, 1}
	},
	{	// "=" ASCII CODE = 61
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0},
		{1, 1, 1, 1, 1},
		{0, 0, 0, 0, 0},
		{1, 1, 1, 1, 1},
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0}
	},
	{	// ">" ASCII CODE = 62
		{1, 0, 0, 0, 0},
		{0, 1, 0, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 0, 1, 0},
		{0, 0, 0, 0, 1},
		{0, 0, 0, 1, 0},
		{0, 0, 1, 0, 0},
		{0, 1, 0, 0, 0},
		{1, 0, 0, 0, 0}
	},
	{	// "?" ASCII CODE = 63
		{0, 1, 1, 1, 0},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{0, 0, 0, 0, 1},
		{0, 0, 0, 1, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 0, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0}
	},
	{	// "@" ASCII CODE = 64
		{0, 1, 1, 1, 0},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 1, 1},
		{1, 0, 1, 0, 1},
		{1, 0, 1, 0, 1},
		{1, 0, 1, 0, 1},
		{1, 0, 0, 1, 1},
		{1, 0, 0, 0, 0},
		{0, 1, 1, 1, 0}
	},
	{	// "A" ASCII CODE = 65
		{0, 0, 1, 0, 0},
		{0, 1, 0, 1, 0},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{1, 1, 1, 1, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1}
	},
	{	// "B" ASCII CODE = 66
		{1, 1, 1, 1, 0},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{1, 1, 1, 1, 0},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{1, 1, 1, 1, 0}
	},
	{	// "C" ASCII CODE = 67
		{0, 1, 1, 1, 0},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 0},
		{1, 0, 0, 0, 0},
		{1, 0, 0, 0, 0},
		{1, 0, 0, 0, 0},
		{1, 0, 0, 0, 0},
		{1, 0, 0, 0, 1},
		{0, 1, 1, 1, 0}
	},
	{	// "D" ASCII CODE = 68
		{1, 1, 1, 0, 0},
		{1, 0, 0, 1, 0},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 1, 0},
		{1, 1, 1, 0, 0}
	},
	{	// "E" ASCII CODE = 69
		{1, 1, 1, 1, 1},
		{1, 0, 0, 0, 0},
		{1, 0, 0, 0, 0},
		{1, 0, 0, 0, 0},
		{1, 1, 1, 1, 0},
		{1, 0, 0, 0, 0},
		{1, 0, 0, 0, 0},
		{1, 0, 0, 0, 0},
		{1, 1, 1, 1, 1}
	},
	{	// "F" ASCII CODE = 70
		{1, 1, 1, 1, 1},
		{1, 0, 0, 0, 0},
		{1, 0, 0, 0, 0},
		{1, 0, 0, 0, 0},
		{1, 1, 1, 1, 0},
		{1, 0, 0, 0, 0},
		{1, 0, 0, 0, 0},
		{1, 0, 0, 0, 0},
		{1, 0, 0, 0, 0}
	},
	{	// "G" ASCII CODE = 71
		{0, 1, 1, 1, 0},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 0},
		{1, 0, 0, 0, 0},
		{1, 0, 1, 1, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{0, 1, 1, 1, 0}
	},
	{	// "H" ASCII CODE = 72
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{1, 1, 1, 1, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1}
	},
	{	// "I" ASCII CODE = 73
		{0, 1, 1, 1, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 1, 1, 1, 0}
	},
	{	// "J" ASCII CODE = 74
		{0, 0, 0, 0, 1},
		{0, 0, 0, 0, 1},
		{0, 0, 0, 0, 1},
		{0, 0, 0, 0, 1},
		{0, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{0, 1, 1, 1, 0}
	},
	{	// "K" ASCII CODE = 75
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 1, 0},
		{1, 0, 1, 0, 0},
		{1, 1, 0, 0, 0},
		{1, 0, 1, 0, 0},
		{1, 0, 0, 1, 0},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1}
	},
	{	// "L" ASCII CODE = 76
		{1, 0, 0, 0, 0},
		{1, 0, 0, 0, 0},
		{1, 0, 0, 0, 0},
		{1, 0, 0, 0, 0},
		{1, 0, 0, 0, 0},
		{1, 0, 0, 0, 0},
		{1, 0, 0, 0, 0},
		{1, 0, 0, 0, 0},
		{1, 1, 1, 1, 1}
	},
	{	// "M" ASCII CODE = 77
		{1, 0, 0, 0, 1},
		{1, 1, 0, 1, 1},
		{1, 1, 0, 1, 1},
		{1, 0, 1, 0, 1},
		{1, 0, 1, 0, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1}
	},
	{	// "N" ASCII CODE = 78
		{1, 0, 0, 0, 1},
		{1, 1, 0, 0, 1},
		{1, 1, 0, 0, 1},
		{1, 0, 1, 0, 1},
		{1, 0, 1, 0, 1},
		{1, 0, 0, 1, 1},
		{1, 0, 0, 1, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1}
	},
	{	// "0" ASCII CODE = 79
		{0, 1, 1, 1, 0},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{0, 1, 1, 1, 0}
	},
	{	// "P" ASCII CODE = 80
		{1, 1, 1, 1, 0},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{1, 1, 1, 1, 0},
		{1, 0, 0, 0, 0},
		{1, 0, 0, 0, 0},
		{1, 0, 0, 0, 0},
		{1, 0, 0, 0, 0}
	},
	{	// "Q" ASCII CODE = 81
		{0, 1, 1, 1, 0},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 1, 0, 1},
		{1, 1, 0, 1, 1},
		{0, 1, 1, 1, 1}
	},
	{	// "R" ASCII CODE = 82
		{1, 1, 1, 1, 0},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{1, 1, 1, 1, 0},
		{1, 0, 0, 1, 0},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1}
	},
	{	// "S" ASCII CODE = 83
		{0, 1, 1, 1, 0},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{0, 1, 0, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 0, 1, 0},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{0, 1, 1, 1, 0}
	},
	{	// "T" ASCII CODE = 84
		{1, 1, 1, 1, 1},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0}
	},
	{	// "U" ASCII CODE = 85
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{0, 1, 1, 1, 0}
	},
	{	// "V" ASCII CODE = 86
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{0, 1, 0, 1, 0},
		{0, 1, 0, 1, 0},
		{0, 1, 0, 1, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0}
	},
	{	// "W" ASCII CODE = 87
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 1, 0, 1},
		{1, 0, 1, 0, 1},
		{1, 0, 1, 0, 1},
		{1, 1, 0, 1, 1},
		{1, 1, 0, 1, 1},
		{0, 1, 0, 1, 0},
		{0, 1, 0, 1, 0}
	},
	{	// "X" ASCII CODE = 88
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{0, 1, 0, 1, 0},
		{0, 1, 0, 1, 0},
		{0, 0, 1, 0, 0},
		{0, 1, 0, 1, 0},
		{0, 1, 0, 1, 0},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1}
	},
	{	// "Y" ASCII CODE = 89
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{0, 1, 0, 1, 0},
		{0, 1, 0, 1, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0}
	},
	{	// "Z" ASCII CODE = 90
		{1, 1, 1, 1, 1},
		{0, 0, 0, 0, 1},
		{0, 0, 0, 1, 0},
		{0, 0, 0, 1, 0},
		{0, 0, 1, 0, 0},
		{0, 1, 0, 0, 0},
		{0, 1, 0, 0, 0},
		{1, 0, 0, 0, 0},
		{1, 1, 1, 1, 1}
	},
	{	// "[" ASCII CODE = 91
		{0, 1, 1, 1, 0},
		{0, 1, 0, 0, 0},
		{0, 1, 0, 0, 0},
		{0, 1, 0, 0, 0},
		{0, 1, 0, 0, 0},
		{0, 1, 0, 0, 0},
		{0, 1, 0, 0, 0},
		{0, 1, 0, 0, 0},
		{0, 1, 1, 1, 0}
	},
	{	// "\" ASCII CODE = 92
		{1, 0, 0, 0, 0},
		{1, 0, 0, 0, 0},
		{0, 1, 0, 0, 0},
		{0, 1, 0, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 0, 1, 0},
		{0, 0, 0, 1, 0},
		{0, 0, 0, 0, 1}
	},
	{	// "]" ASCII CODE = 93
		{0, 1, 1, 1, 0},
		{0, 0, 0, 1, 0},
		{0, 0, 0, 1, 0},
		{0, 0, 0, 1, 0},
		{0, 0, 0, 1, 0},
		{0, 0, 0, 1, 0},
		{0, 0, 0, 1, 0},
		{0, 0, 0, 1, 0},
		{0, 1, 1, 1, 0}
	},
	{	// "^" ASCII CODE = 94
		{0, 0, 1, 0, 0},
		{0, 1, 0, 1, 0},
		{1, 0, 0, 0, 1},
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0}
	},
	{	// "_" ASCII CODE = 95
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0},
		{1, 1, 1, 1, 1}
	},
	{	// "`" ASCII CODE = 96
		{0, 1, 0, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 0, 1, 0},
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0}
	},
	{	// "a" ASCII CODE = 97
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0},
		{0, 1, 1, 1, 0},
		{1, 0, 0, 0, 1},
		{0, 0, 0, 0, 1},
		{0, 1, 1, 1, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 1, 1},
		{0, 1, 1, 0, 1}
	},
	{	// "b" ASCII CODE = 98
		{1, 0, 0, 0, 0},
		{1, 0, 0, 0, 0},
		{1, 0, 1, 1, 0},
		{1, 1, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{0, 1, 1, 1, 0}
	},
	{	// "c" ASCII CODE = 99
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0},
		{0, 1, 1, 1, 0},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 0},
		{1, 0, 0, 0, 0},
		{1, 0, 0, 0, 0},
		{1, 0, 0, 0, 1},
		{0, 1, 1, 1, 0}
	},
	{	// "d" ASCII CODE = 100
		{0, 0, 0, 0, 1},
		{0, 0, 0, 0, 1},
		{0, 1, 1, 0, 1},
		{1, 0, 0, 1, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{0, 1, 1, 1, 1}
	},
	{	// "e" ASCII CODE = 101
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0},
		{0, 1, 1, 1, 0},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{1, 1, 1, 1, 1},
		{1, 0, 0, 0, 0},
		{1, 0, 0, 0, 1},
		{0, 1, 1, 1, 0}
	},
	{	// "f" ASCII CODE = 102
		{0, 0, 0, 1, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{1, 1, 1, 1, 1},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 1, 0, 0, 0}
	},
	{	// "g" ASCII CODE = 103
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0},
		{0, 1, 1, 1, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{0, 1, 1, 1, 1},
		{0, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{0, 1, 1, 1, 0}
	},
	{	// "h" ASCII CODE = 104
		{1, 0, 0, 0, 0},
		{1, 0, 0, 0, 0},
		{1, 0, 1, 1, 0},
		{1, 1, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1}
	},
	{	// "i" ASCII CODE = 105
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 0, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0}
	},
	{	// "j" ASCII CODE = 106
		{0, 0, 0, 0, 1},
		{0, 0, 0, 0, 1},
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 1},
		{0, 0, 0, 0, 1},
		{0, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{0, 1, 1, 1, 0}
	},
	{	// "k" ASCII CODE = 107
		{1, 0, 0, 0, 0},
		{1, 0, 0, 0, 0},
		{1, 0, 0, 1, 0},
		{1, 0, 1, 0, 0},
		{1, 1, 0, 0, 0},
		{1, 1, 0, 0, 0},
		{1, 0, 1, 0, 0},
		{1, 0, 0, 1, 0},
		{1, 0, 0, 0, 1}
	},
	{	// "l" ASCII CODE = 108
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 1, 0, 0, 0}
	},
	{	// "m" ASCII CODE = 109
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0},
		{1, 1, 0, 1, 0},
		{1, 0, 1, 0, 1},
		{1, 0, 1, 0, 1},
		{1, 0, 1, 0, 1},
		{1, 0, 1, 0, 1},
		{1, 0, 1, 0, 1},
		{1, 0, 1, 0, 1}
	},
	{	// "n" ASCII CODE = 110
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0},
		{1, 1, 1, 1, 0},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1}
	},
	{	// "o" ASCII CODE = 111
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0},
		{0, 1, 1, 1, 0},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{0, 1, 1, 1, 0}
	},
	{	// "p" ASCII CODE = 112
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0},
		{1, 0, 1, 1, 0},
		{1, 1, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{1, 1, 1, 1, 0},
		{1, 0, 0, 0, 0},
		{1, 0, 0, 0, 0}
	},
	{	// "q" ASCII CODE = 113
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0},
		{0, 1, 1, 0, 1},
		{1, 0, 0, 1, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{0, 1, 1, 1, 1},
		{0, 0, 0, 0, 1},
		{0, 0, 0, 0, 1}
	},
	{	// "r" ASCII CODE = 114
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0},
		{1, 0, 1, 1, 0},
		{1, 1, 0, 0, 1},
		{1, 0, 0, 0, 0},
		{1, 0, 0, 0, 0},
		{1, 0, 0, 0, 0},
		{1, 0, 0, 0, 0},
		{1, 0, 0, 0, 0}
	},
	{	// "s" ASCII CODE = 115
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0},
		{0, 1, 1, 1, 1},
		{1, 0, 0, 0, 0},
		{1, 0, 0, 0, 0},
		{0, 1, 1, 1, 0},
		{0, 0, 0, 0, 1},
		{0, 0, 0, 0, 1},
		{1, 1, 1, 1, 0}
	},
	{	// "t" ASCII CODE = 116
		{0, 0, 0, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{1, 1, 1, 1, 1},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 0, 1, 1}
	},
	{	// "u" ASCII CODE = 117
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 1, 1},
		{0, 1, 1, 0, 1}
	},
	{	// "v" ASCII CODE = 118
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{0, 1, 0, 1, 0},
		{0, 1, 0, 1, 0},
		{0, 1, 0, 1, 0},
		{0, 0, 1, 0, 0}
	},
	{	// "w" ASCII CODE = 119
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 1, 0, 1},
		{1, 0, 1, 0, 1},
		{1, 0, 1, 0, 1},
		{1, 0, 1, 0, 1},
		{0, 1, 0, 1, 0}
	},
	{	// "x" ASCII CODE = 120
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{0, 1, 0, 1, 0},
		{0, 0, 1, 0, 0},
		{0, 1, 0, 1, 0},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1}
	},
	{	// "y" ASCII CODE = 121
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{0, 1, 0, 1, 0},
		{0, 1, 0, 1, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0}
	},
	{	// "z" ASCII CODE = 122
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0},
		{1, 1, 1, 1, 1},
		{0, 0, 0, 0, 1},
		{0, 0, 0, 1, 0},
		{0, 0, 1, 0, 0},
		{0, 1, 0, 0, 0},
		{1, 0, 0, 0, 0},
		{1, 1, 1, 1, 1}
	},
	{	// "{" ASCII CODE = 123
		{0, 0, 1, 1, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 1, 0, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 1, 0}
	},
	{	// "|" ASCII CODE = 124
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0}
	},
	{	// "}" ASCII CODE = 125
		{0, 1, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 0, 1, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 1, 1, 0, 0}
	},
	{	// "~" ASCII CODE = 126
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0},
		{0, 1, 0, 0, 0},
		{1, 0, 1, 0, 1},
		{0, 0, 0, 1, 0},
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0}
	},
	{	// "■" ASCII CODE = 範囲外
		{1, 1, 1, 1, 1},
		{1, 1, 1, 1, 1},
		{1, 1, 1, 1, 1},
		{1, 1, 1, 1, 1},
		{1, 1, 1, 1, 1},
		{1, 1, 1, 1, 1},
		{1, 1, 1, 1, 1},
		{1, 1, 1, 1, 1},
		{1, 1, 1, 1, 1}
	}
};

