//! @file FrameGraphics.hh
//! @brief フレームバッファグラフィックスクラス
//!
//! 画面上に図等の描画を行います。
//! ただし，/dev/fb* が必要で，ディスプレイの色深度は16bitか32bitのみ対応。gccとフレームバッファさえあれば動く。
//!
//! @date 2018/01/07
//! @author Yuki YOKOKURA
//
// Copyright (C) 2011-2019 Yuki YOKOKURA
// This program is free software;
// you can redistribute it and/or modify it under the terms of the FreeBSD License.
// For details, see the License.txt file.

#ifndef FRAMEGRAPHICS
#define FRAMEGRAPHICS

#include <linux/fb.h>
#include <string>

namespace ARCS {	// ARCS名前空間
	class FrameGraphics {
		// フレームバッファクラス
		public:
			// 色の定義  注意!!：const int*_t ColorSet* の定義と相違が無いようにせよ！
			enum FGcolors {
				RED,	// 赤
				GREEN,	// 緑
				BLUE,	// 青
				CYAN,	// 水(シアン)
				MAGENTA,// 紫(マゼンタ)
				YELLOW,	// 黄(イエロー)
				ORANGE,	// 橙
				WHITE,	// 白
				GRAY75,	// 灰 輝度75％
				GRAY50,	// 灰 輝度50％
				GRAY25,	// 灰 輝度25％
				BLACK	// 黒
			};
			// 文字列揃え位置の定義
			enum FGalign {
				ALIGN_LEFT,		// 文字列左揃え
				ALIGN_CENTER,	// 文字列中央揃え
				ALIGN_RIGHT		// 文字列右揃え
			};
			explicit FrameGraphics(const char* device);						// コンストラクタ フレームバッファ初期化
			FrameGraphics(FrameGraphics&& r);								//!< ムーブコンストラクタ
			~FrameGraphics(void);											// デストラクタ   フレームバッファ開放
			void ShowParam(void);											// 画面情報の表示
			void DrawPoint(int x, int y, FGcolors color);					// フレームバッファに点(x,y)を描画する
			void DrawPoint(int x, int y, double r, double g, double b);		// フレームバッファに点(x,y)を描画する
			void DrawLine(int x1, int y1, int x2, int y2, FGcolors color);				// 直線の描画
			void DrawLine(int x1, int y1, int x2, int y2, double r, double g, double b);// 直線の描画
			void DrawRect(int x, int y, int w, int h, FGcolors color);					// 長方形の描画(色は塗らない)
			void DrawRect(int x, int y, int w, int h, double r, double g, double b);	// 長方形の描画(色は塗らない)
			void DrawRectFill(int x, int y, int w, int h, FGcolors color);				// 長方形の描画(色を塗る)
			void DrawRectFill(int x, int y, int w, int h, double r, double g, double b);// 長方形の描画(色を塗る)
			void DrawCircle(int cx, int cy, int radius, unsigned int N, FGcolors color);// 円の描画をする cx,cy：円の中心座標，radius：半径，N：円の分割数
			void DrawCircle(int cx, int cy, int radius, unsigned int N, double r, double g, double b);// 円の描画をする cx,cy：円の中心座標，radius：半径，N：円の分割数
			void ClearScreen(void);										// 画面の消去
			void ClearRect(int x, int y, int w, int h);					// 長方形の範囲内を消去
			int GetWidth(void) const;									// 画面の幅の取得
			int GetHeight(void) const;									// 画面の高さの取得
			int GetDepth(void) const;									// 画面の色深度の取得
			void PrintValue(int x, int y, FGalign align, std::string format, double val);// 指定した書式で文字列を描画する x：[px] 横位置，y：[px] 縦位置，align：揃え位置，format：書式指定子，val：所望の数値 formatの書式指定子は printf関数 の場合の書き方と同等
			void PrintText(int x, int y, FGalign align, std::string text);// 文字列を描画する x：[px]横位置，y：[px] 縦位置，align：揃え位置，text：所望の文字列
			void WriteFont(int x, int y, unsigned int ascii);			// 1文字分のフォントデータをフレームバッファに書き込む x：[px] 横位置，y：[px] 縦位置，ascii：アスキーコード
			void PrepareFontData(FGcolors fore_color, FGcolors back_color);	// 設定した色のフォントデータを準備
			void PrintAllFontData();	// 収録されているすべてのフォントデータを描画するテスト用関数
		
		private:
			FrameGraphics(const FrameGraphics&) = delete;					// コピーコンストラクタ使用禁止
			const FrameGraphics& operator=(const FrameGraphics&) = delete;	// 代入演算子使用禁止
			
			// フレームバッファ関連
			bool isEnabled;		// フレームバッファ有効/無効
			int FBfd;			// フレームバッファ ファイルディスクリプタ
			int16_t *FBptr16;	// フレームバッファ ポインタ 色深度16bit用
			int32_t *FBptr32;	// フレームバッファ ポインタ 色深度32bit用
			struct fb_fix_screeninfo finfo;	// 固定情報構造体
			struct fb_var_screeninfo vinfo;	// 可変情報構造体
			int width;		// [px]		横幅
			int height;		// [px]		高さ
			int depth;		// [bits]	色深度
			long size;		// [bytes]	画面の大きさ
			long length;	// 			フレームバッファポインタの長さ
			int xofst;		// xオフセット
			int yofst;		// yオフセット
			int xlen;		// x方向長さ
			int bppx;		// [bit/px]	ピクセル当りのビット数
			
			// 色の定義
			static const unsigned int NUM_COLOR_SET = 12;	// 定義する色の数  注意!!：enum FGcolors の要素数と同一にせよ！
			static const int16_t ColorSet16[NUM_COLOR_SET];	// 色定義(16bit色深度用)
			static const int32_t ColorSet32[NUM_COLOR_SET];	// 色定義(32bit色深度用)
			
			// フォントの定義
			static const unsigned int FONT_NUM     = 96;// 収録しているフォントの数
			static const unsigned int FONT_WIDTH   = 5;	// [px] フォントの幅
			static const unsigned int FONT_HEIGHT  = 9;	// [px] フォントの高さ
			static const unsigned int FONT_FST_ASCII =  32;	// フォントデータに収録している最初の文字のアスキーコード
			static const unsigned int FONT_END_ASCII = 126;	// フォントデータに収録している最後の文字のアスキーコード
			static const int FONT_DATA[FONT_NUM][FONT_HEIGHT][FONT_WIDTH];// フォントデータ
			int16_t FontPrepared16[FONT_NUM][FONT_HEIGHT][FONT_WIDTH];	// 準備済みのフォントデータ(32bit色深度用)
			int32_t FontPrepared32[FONT_NUM][FONT_HEIGHT][FONT_WIDTH];	// 準備済みのフォントデータ(32bit色深度用)
			bool IsFontDataLocked;	// フォントデータスピンロックフラグ
			
			// 文字列関連の定義
			static const unsigned int TEXT_INTERVAL = 1;	// [px] 文字の間隔
			static const unsigned int TEXT_MAXLEN = 256;	// 文字列の最大値
			
			// 色変換関連の関数
			static int16_t RGBcolorToData16(double Red, double Green, double Blue);	// 0～1の浮動小数点数値を赤緑青色の32bit輝度データに変える
			static int16_t DoubleToIntensity16(double u);		// 0～1の浮動小数点数値を白色の32bit輝度データに変える
			static int16_t DoubleToRedIntensity16(double u);	// 0～1の浮動小数点数値を赤色の32bit輝度データに変える
			static int16_t DoubleToGreenIntensity16(double u);	// 0～1の浮動小数点数値を緑色の32bit輝度データに変える
			static int16_t DoubleToBlueIntensity16(double u);	// 0～1の浮動小数点数値を青色の32bit輝度データに変える
			static int32_t RGBcolorToData32(double Red, double Green, double Blue);	// 0～1の浮動小数点数値を赤緑青色の32bit輝度データに変える
			static int32_t DoubleToIntensity32(double u);		// 0～1の浮動小数点数値を白色の32bit輝度データに変える
			static int32_t DoubleToRedIntensity32(double u);	// 0～1の浮動小数点数値を赤色の32bit輝度データに変える
			static int32_t DoubleToGreenIntensity32(double u);	// 0～1の浮動小数点数値を緑色の32bit輝度データに変える
			static int32_t DoubleToBlueIntensity32(double u);	// 0～1の浮動小数点数値を青色の32bit輝度データに変える
		
	};
}

#endif

