//! @file SFmultthread.cc
//! @brief 実時間マルチスレッド生成・破棄クラス(SCHED_FIFO版)
//! @date 2019/01/17
//! @author Yuki YOKOKURA
//
// Copyright (C) 2011-2019 Yuki YOKOKURA
// This program is free software;
// you can redistribute it and/or modify it under the terms of the BSD License.
// For details, see the License.txt file.

#include "SFmultthread.hh"
#include "ARCSeventlog.hh"

using namespace ARCS;

//! @brief コンストラクタ
//! @param[in]	ThreadNum		実時間スレッドの数
//! @param[in]	PeriodicFuncs	周期関数への関数ポインタ
//! @param[in]	FuncArgs		周期関数の引数
//! @param[in]	PeriodicTime	制御周期[ns]
//! @param[in]	CPUNumber		使用するCPUコアの数
SFmultthread::SFmultthread(const unsigned int ThreadNum, void (** const PeriodicFuncs)(void*), void* const FuncArgs, const unsigned long* PeriodicTime, const unsigned int* CPUNumber)
	: NumOfThread(ThreadNum)	// 動作したいスレッドの数
{
	PassedLog();	// イベントログにココを通過したことを記録
	// 個々の実時間スレッドを生成
	for(unsigned int i=0;i<NumOfThread;i++){
		pSFthread[i] = new SFthread(PeriodicTime[i], PeriodicFuncs[i], FuncArgs, CPUNumber[i]);	// スレッド生成
	}
	PassedLog();	// イベントログにココを通過したことを記録
}

//! @brief デストラクタ
SFmultthread::~SFmultthread(){
	PassedLog();	// イベントログにココを通過したことを記録
	for(unsigned int i=0;i<NumOfThread;i++)delete pSFthread[i];	// すべての実時間スレッドの消去
	PassedLog();	// イベントログにココを通過したことを記録
}

//! @brief 実時間スレッドを開始させる関数
void SFmultthread::Start(void){
	for(unsigned int i=0;i<NumOfThread;i++)pSFthread[i]->Start();		// すべての実時間スレッドを開始
	for(unsigned int i=0;i<NumOfThread;i++)pSFthread[i]->WaitStart();	// すべての実時間スレッド開始処理が完了するまで待機(ブロッキング動作)
}

//! @brief 実時間スレッドを終了させる関数
void SFmultthread::Stop(void){
	for(unsigned int i=0;i<NumOfThread;i++)pSFthread[i]->Stop();	// すべての実時間スレッドを停止
	for(unsigned int i=0;i<NumOfThread;i++)pSFthread[i]->WaitStop();// すべての実時間スレッド終了処理が完了するまで待機(ブロッキング動作)
}

//! @brief 実時間スレッドを緊急停止させる関数
void SFmultthread::EmergencyStop(void){
	for(unsigned int i=0;i<NumOfThread;i++)pSFthread[i]->Stop();	// すべての実時間スレッドを停止
	// 終了待機はせずに関数を抜ける(非ブロッキング動作)
}

//! @brief 経過時間，測定した制御周期，計算に消費した時間，最大値，最小値を取得する関数(単位は秒)
//! @param[out]	Time			経過時間
//! @param[out]	PeriodicTime	制御周期
//! @param[out]	ComputationTime	消費時間
//! @param[out]	MaxTime			最大周期
//! @param[out]	MinTime			最小周期
void SFmultthread::GetTimeValue(
		volatile double *Time,
		volatile double *PeriodicTime, 
		volatile double *ComputationTime, 
		volatile double *MaxTime, 
		volatile double *MinTime
	){
	*Time = pSFthread[0]->GetTime();	// [s]	最優先実時間スレッドの時刻の取得
	
	// 各実時間スレッドの周期時間と消費時間の取得
	for(unsigned int i=0;i<NumOfThread;i++){
		PeriodicTime[i] = pSFthread[i]->GetSmplTime();		// [s] 実際に測定した制御周期
		ComputationTime[i] = pSFthread[i]->GetCompTime();	// [s] 制御演算のために消費した時間
		MaxTime[i] = pSFthread[i]->GetMaxTime();			// [s] 最大値
		MinTime[i] = pSFthread[i]->GetMinTime();			// [s] 最小値
	}
}

