//! @file SFthread.cc
//! @brief 実時間スレッド生成・破棄クラス(SCHED_FIFO版)
//!        実時間スレッドを生成・破棄する
//!        実際に計測された制御周期や計算消費時間も提供する
//! @date 2019/01/17
//! @author Yuki YOKOKURA
//
// Copyright (C) 2011-2019 Yuki YOKOKURA
// This program is free software;
// you can redistribute it and/or modify it under the terms of the FreeBSD License.
// For details, see the License.txt file.

#include <time.h>
#include <unistd.h>
#include <math.h>
#include "SFthread.hh"
#include "ARCSeventlog.hh"

using namespace ARCS;

SFthread::SFthread(const unsigned long Time, void (* const RTfunc)(void*), void* const Arg, int CPUno)
	// コンストラクタ
	// Time：制御周期，RTfunc：制御用実行関数，Arg：制御用実行関数の引数，CPUno：使用するCPUコアの番号
	: StateFlag(SFID_STOP),	// 動作状態フラグを「停止状態」に設定
	  Ts(Time),				// [ns] 制御周期の格納
	  pRTfunc(RTfunc),		// 制御用実行関数への関数ポインタを格納
	  pRTarg(Arg),			// 関数ポインタの引数の格納
	  LoopCount(0),			// 制御ループカウンタの初期化
	  ActPeriodicTime(),	// 周期時間の初期化
	  ComputationTime(),	// 消費時間の初期化
	  ThreadID(0),			// スレッド識別子の初期化
	  ThreadParam(),		// スレッドパラメータ
	  MaxMemo(0),			// サンプリング時間最大値計算用
	  MinMemo(Time*1e-9)	// サンプリング時間最小値計算用
{
	// 実時間スレッドの生成と優先度の設定
	PassedLog();	// イベントログにココを通過したことを記録
	ThreadParam.sched_priority = sched_get_priority_max(SCHED_FIFO);		// 可能な限り高いスレッド優先度を取得(0～99)
	pthread_create(&ThreadID, NULL, (void*(*)(void*))RealTimeThread, this);	// スレッド生成
	pthread_setschedparam(ThreadID, SCHED_FIFO, &ThreadParam);				// スレッド優先度を可能な限り高く設定
	
	// CPUコアの割り当て
	cpu_set_t cpuset;		// CPU設定用変数
	CPU_ZERO(&cpuset);		// まずCPU設定用変数をクリアして，
	CPU_SET(CPUno,&cpuset);	// 使用するCPUコア番号をCPU設定用変数にセットし，
	pthread_setaffinity_np(ThreadID, sizeof(cpu_set_t), &cpuset);	// 実時間スレッドを指定のCPUコアに割り当てる
	PassedLog();	// イベントログにココを通過したことを記録
}

SFthread::~SFthread(){
	// デストラクタ
	PassedLog();	// イベントログにココを通過したことを記録
	pthread_cancel(ThreadID);		// 実時間スレッド終了＆破棄
	PassedLog();	// イベントログにココを通過したことを記録
}

void SFthread::RealTimeThread(SFthread *p){
	// 実時間スレッド
	
	// 時間関連の構造体
	timespec NextTime = {0};		// お休み時間設定用
	timespec TimeInWait = {0};		// 待機ループ内で取得する現在時刻
	timespec PeriodTime = p->nsec_to_timespec(p->Ts);	// 所望の制御周期
	timespec StartTime = {0};		// 開始時刻格納用
	timespec StartTimePrev = {0};	// 前回の開始時間格納用
	timespec EndTime = {0};			// 終了時刻格納用
	timespec PreventStuck = {0};	// 「BUG: soft lockup - CPU#0 Stuck for 67s!」を回避するためのスリープ用
	
	// 動作状態が「開始」に設定されるまで待機
	while(p->StateFlag != SFID_START){
		usleep(WAIT_TIME);		// 適当に待機
	}
	p->StateFlag = SFID_RUN;	// 動作状態フラグを「動作中」に設定
	
	clock_gettime(CLOCK_MONOTONIC, &NextTime);				// 初期開始時刻の取得
	StartTimePrev = p->timespec_sub(NextTime, PeriodTime);	// 実際の制御周期計算用の初期値設定
	
	// 実時間ループ
	while(p->StateFlag != SFID_STOP){	// 動作状態フラグが「停止」に設定されるまでループ
		// ここからリアルタイム空間
		clock_gettime(CLOCK_MONOTONIC, &StartTime);				// 開始時刻の取得
		
		p->pRTfunc(p->pRTarg);	// 制御用関数の実行(関数ポインタにより、ここで実際の制御関数が呼ばれる)
		
		p->LoopCount++;			// 制御ループカウンタをインクリメント
		p->ActPeriodicTime = p->timespec_sub(StartTime, StartTimePrev);	// 実際の周期時間を計算(timespec構造体は単純に減算できないことに注意)
		StartTimePrev = StartTime;										// 次回用に今回の開始時刻を格納
		NextTime = p->timespec_add(NextTime, PeriodTime);				// 開始時刻に制御周期を加算して次の時刻を計算
		clock_nanosleep(CLOCK_MONOTONIC, 0, &PreventStuck, NULL);		// 「BUG: soft lockup - CPU#0 Stuck for 67s!」を回避するためのスリープ
		clock_gettime(CLOCK_MONOTONIC, &EndTime);						// 終了時刻の取得
		p->ComputationTime = p->timespec_sub(EndTime, StartTime);		// 消費時間を計算(timespec構造体は単純に減算できないことに注意)
		
		// 次の時刻になるまで待機
		// clock_nanosleep関数を使うとジッタが酷すぎてお話にならないので独自に実装
		// (スレッドを休ませないことが肝。ただし、“割り当てられたCPUコアにおいて” 実時間スレッド以外のタスクはほぼ実行できなくなる諸刃の剣。)
		while(p->StateFlag != SFID_STOP){
			clock_gettime(CLOCK_MONOTONIC, &TimeInWait);	// 現在時刻の取得
			if(p->timespec_lessthaneq(NextTime, TimeInWait) == true){
				// 現在時刻と予め計算した次の時刻とを比較して，超えたら待機終了
				break;
			}
		}
		// 一般的には，下記のように clock_nanosleep関数で待機するが，
		// 100μsの制御周期で動かすにはジッタが大きすぎて使いものにならない。
		// 「clock_nanosleep(CLOCK_MONOTONIC, TIMER_ABSTIME, &NextTime, NULL);」
		
		// リアルタイム空間ここまで
	}
	
	p->StateFlag = SFID_EXCMPL;			// 動作状態フラグを「終了動作完了」に設定
}

void SFthread::Start(void){
	// スレッド実行を開始する関数
	SendState(SFID_START);
}

void SFthread::WaitStart(void) const {
	// スレッド実行が開始されるまで待機する関数
	while(1){
		if(ReadState() == SFID_RUN) break;	// 開始動作が完了するまでブロッキング
		usleep(WAIT_TIME);
	}
}

void SFthread::Stop(void){
	// スレッド実行を停止する関数
	SendState(SFID_STOP);
}

void SFthread::WaitStop(void) const {
	// スレッド実行が停止されるまで待機する関数
	while(1){
		if(ReadState() == SFID_EXCMPL) break;	// 終了動作が完了するまでブロッキング
		usleep(WAIT_TIME);
	}
}

void SFthread::SendState(const enum ThreadState State){
	// 動作状態を書き込む
	// State；所望の動作状態
	StateFlag = State;
}

enum SFthread::ThreadState SFthread::ReadState(void) const {
	// 動作状態を読み出す
	// 戻り値；現在の動作状態
	return StateFlag;
}

double SFthread::GetTime(void) const {
	// 時刻を取得する関数
	return (double)LoopCount*Ts*1e-9;		// [s] 時刻に変換して返す
}

double SFthread::GetSmplTime(void){
	// 計測された実際のサンプリング時間を取得する関数
	return timespec_to_sec(ActPeriodicTime);// [s] 時刻に変換して返す
}

double SFthread::GetCompTime(void){
	// 計測された消費時間を取得する関数
	return timespec_to_sec(ComputationTime);// [s] 時刻に変換して返す
}

double SFthread::GetMaxTime(void){
	// 計測された実際のサンプリング時間の最大値を取得する関数
	double TsZ0 = GetSmplTime();	// 今のサンプリング時間を取得
	if(MaxMemo < TsZ0){
		MaxMemo = TsZ0;
	}
	return MaxMemo;
}

double SFthread::GetMinTime(void){
	// 計測された実際のサンプリング時間の最小値を取得する関数
	double TsZ0 = GetSmplTime();	// 今のサンプリング時間を取得
	if(TsZ0 < MinMemo && 1e-6 < TsZ0){
		MinMemo = TsZ0;
	}
	return MinMemo;
}

timespec SFthread::timespec_add(timespec a, timespec b){
	// timespec構造体用の時刻を加算する関数 a + b
	timespec ret;
	ret.tv_sec = a.tv_sec + b.tv_sec;		// aとbの秒カウンタを加算
	ret.tv_nsec = a.tv_nsec + b.tv_nsec;	// aとbのナノ秒カウンタを加算
	if(ONE_SEC_IN_NANO <= ret.tv_nsec){
		// ナノ秒カウンタが正方向に溢れる場合
		ret.tv_nsec -= ONE_SEC_IN_NANO;	// ナノ秒カウンタから1秒分引いて，
		ret.tv_sec++;					// 代わりに秒カウンタを加算
	}
	return ret;
}

timespec SFthread::timespec_sub(timespec a, timespec b){
	// timespec構造体用の時刻を減算する関数 a - b
	timespec ret;
	ret.tv_sec = a.tv_sec - b.tv_sec;		// aとbの秒カウンタを減算
	ret.tv_nsec = a.tv_nsec - b.tv_nsec;	// aとbのナノ秒カウンタを減算
	if(ret.tv_nsec < 0){
		// ナノ秒カウンタが負方向に溢れる場合
		ret.tv_nsec += ONE_SEC_IN_NANO;	// ナノ秒カウンタに1秒分加えて，
		ret.tv_sec--;					// 代わりに秒カウンタを減算
	}
	return ret;
}

bool SFthread::timespec_lessthaneq(timespec a, timespec b){
	// timespec構造体用の時刻を比較する関数 a <= b
	if(a.tv_sec != b.tv_sec){
		if(a.tv_sec < b.tv_sec){
			return true;
		}else{
			return false;
		}
	}else{
		if(a.tv_nsec <= b.tv_nsec){
			return true;
		}else{
			return false;
		}
	}
}

timespec SFthread::nsec_to_timespec(unsigned long time){
	// ナノ秒からtimespec構造体へ変換する関数
	timespec ret;
	ret.tv_sec = time/ONE_SEC_IN_NANO;
	ret.tv_nsec = time - ret.tv_sec*ONE_SEC_IN_NANO;
	return ret;
}

double SFthread::timespec_to_sec(timespec time){
	// timespec構造体から秒へ変換する関数
	return (double)time.tv_sec + (double)time.tv_nsec*1e-9;	// [s] 時刻に変換して返す
}

