//! @file SFthread.hh
//! @brief 実時間スレッド生成・破棄クラス(SCHED_FIFO版)
//!        実時間スレッドを生成・破棄する
//!        実際に計測された制御周期や計算消費時間も提供する
//! @date 2018/07/25
//! @author Yuki YOKOKURA
//
// Copyright (C) 2011-2018 Yuki YOKOKURA
// This program is free software;
// you can redistribute it and/or modify it under the terms of the FreeBSD License.
// For details, see the License.txt file.

#ifndef SFTHREADING
#define SFTHREADING

#include <pthread.h>
#include <sys/time.h>

typedef void (*SFfuncPtr)(void*);	// 関数ポインタ定義

namespace ARCS {	// ARCS名前空間
	class SFthread {
		// 実時間スレッド生成・破棄クラス
		public:
			// 動作状態の定義
			enum ThreadState {
				SFID_ERROR,	// エラー検出
				SFID_START,	// 開始
				SFID_RUN,	// 動作中
				SFID_STOP,	// 停止
				SFID_EXCMPL	// 終了動作完了
			};
			// コンストラクタ Time：制御周期，RTfunc：制御用実行関数，Arg：制御用実行関数の引数，CPUno：使用するCPUコアの番号
			SFthread(const unsigned long Time, void (* const RTfunc)(void*), void* const Arg, int CPUno);
			~SFthread();			// デストラクタ
			void Start(void);			// スレッド実行を開始する関数
			void WaitStart(void) const;	// スレッド実行が開始されるまで待機する関数
			void Stop(void);			// スレッド実行を停止する関数
			void WaitStop(void) const;	// スレッド実行が停止されるまで待機する関数
			void SendState(const enum ThreadState State);	// 動作状態を書き込む State；所望の動作状態
			enum ThreadState ReadState(void) const;			// 動作状態を読み出す 戻り値；現在の動作状態
			double GetTime(void) const;	// [s] 時刻を取得する関数
			double GetSmplTime(void);	// [s] 計測された実際のサンプリング時間を取得する関数
			double GetCompTime(void);	// [s] 計測された消費時間を取得する関数
			double GetMaxTime(void);	// [s] 計測された実際のサンプリング時間の最大値を取得する関数
			double GetMinTime(void);	// [s] 計測された実際のサンプリング時間の最小値を取得する関数
			
		private:
			static const unsigned int WAIT_TIME = 1000;		// [μs] スレッド開始＆終了待機ループのポーリング周期
			static const long ONE_SEC_IN_NANO = 1000000000;	// [ns] 1秒をナノ秒で表すと
			
			SFthread(const SFthread&);					// コピーコンストラクタ使用禁止
			const SFthread& operator=(const SFthread&);	// 代入演算子使用禁止
			volatile enum ThreadState StateFlag;		// 動作状態フラグ
			const unsigned long Ts;			// 制御周期
			void (* const pRTfunc)(void*);	// 制御用実行関数へのポインタ
			void* const pRTarg;				// 関数ポインタの引数
			unsigned long LoopCount;		// 制御ループカウンタ ( 稼働できる時間 = Ts*2^(sizeof(unsigned long)*8) = 100e-6*2^32 = 約119時間(例) ) 稼働時間を伸ばしたければ「unsigned long long」にする
			timespec ActPeriodicTime;		// 計測された実際の周期時間
			timespec ComputationTime;		// 計算によって消費された時間 (つまり ComputationTime < ActPeriodicTime でなければならない)
			static void RealTimeThread(SFthread *p);	// 実時間スレッド
			pthread_t ThreadID;				// スレッド識別子
			struct sched_param ThreadParam;	// スレッドパラメータ
			double MaxMemo;					// サンプリング時間最大値計算用
			double MinMemo;					// サンプリング時間最小値計算用

			static timespec timespec_add(timespec a, timespec b);	// timespec構造体用の時刻を加算する関数 a + b
			static timespec timespec_sub(timespec a, timespec b);	// timespec構造体用の時刻を減算する関数 a - b
			static bool timespec_lessthaneq(timespec a, timespec b);// timespec構造体用の時刻を比較する関数 a <= b
			static timespec nsec_to_timespec(unsigned long time);	// ナノ秒からtimespec構造体へ変換する関数
			static double timespec_to_sec(timespec time);			// timespec構造体から秒へ変換する関数
	};
}

#endif

