//! @file ControlFunctions.cc
//! @brief 制御用周期実行関数群クラス
//! @date 2019/02/26
//! @author Yuki YOKOKURA
//
// Copyright (C) 2011-2019 Yuki YOKOKURA
// This program is free software;
// you can redistribute it and/or modify it under the terms of the BSD License.
// For details, see the License.txt file.

#include <cmath>
#include "ControlFunctions.hh"
#include "ScreenParams.hh"
#include "Interface.hh"
#include "GraphPlot.hh"
#include "ARCSprint.hh"
#include "ARCSassert.hh"
#include "Matrix.hh"
#include "SquareWave.hh"
#include "MotorSimulator.hh"
#include "DisturbanceObsrv.hh"

using namespace ARCS;

//! @brief スレッド間グローバル変数用の無名名前空間
namespace {
	double t = 0;	//!< [s]	時刻 (ControlFunction1を基準とする)
	std::array<double, ConstParams::ACTUATOR_MAX> CurrentRef  = {0};//!< [A]   q軸電流指令
	std::array<double, ConstParams::ACTUATOR_MAX> PositionRes = {0};//!< [rad] 位置応答
	std::array<double, ConstParams::ACTUATOR_MAX> TorqueRes = {0};	//!< [Nm]  トルク応答
	std::array<double, ConstParams::ACTUATOR_MAX> CurrentRes = {0};	//!< [A]   q軸電流応答
}

//! @brief 制御用周期実行関数1
//! @param[in]	pCF	クラスメンバアクセス用ポインタ
void ControlFunctions::ControlFunction1(ControlFunctions* const pCF){
	// 制御用定数設定
	const double Ts = ConstParams::SAMPLING_TIME[0]*1e-9;	// [s]		制御周期
	const double Kt = 0.001;	// [Nm/A] 		トルク定数
	const double Jm = 1e-4;		// [kgm^2]		モータ慣性
	const double Dm = 1e-4;		// [Nm/(rad/s)]	モータ粘性
	const double gdis = 300;	// [rad/s]		外乱オブザーバの推定帯域
	const double Kp_ini = 3;	// [-]			速度制御比例ゲイン初期値
	
	// 制御用変数宣言
	static double iqref = 0.1;	// [A]		q軸電流指令
	static double tload = 0;	// [Nm]		負荷トルク
	static double tdis  = 0;	// [Nm]		外乱トルク
	static double wm    = 0;	// [rad/s]	モータ速度
	static double wmref = 0;	// [rad/s]	モータ速度指令
	static double Kp    = 0;	// [-]		速度制御比例ゲイン
	
	// 制御器等々の宣言
	static MotorSimulator Motor1(Kt, Jm, Dm, Ts);						// モータシミュレータ
	static DisturbanceObsrv<DObType::FULL_1ST> DOb1(Kt, Jm, gdis, Ts);	// 外乱オブザーバ
	
	if(pCF->CmdFlag == CTRL_INIT){
		// 初期化モード (ここは制御開始時/再開時に1度だけ呼び出される(非リアルタイム空間なので重い処理もOK))
		ScreenParams::InitOnlineSetVar(Kp_ini);	// オンライン設定変数の初期値の設定
		Interface::ServoON();			// サーボON指令の送出
		Motor1.ClearStateVector();		// モータシミュレータの状態ベクトルのクリア (コーディングの例)
	}
	if(pCF->CmdFlag == CTRL_LOOP){
		// 周期モード (ここは制御周期 SAMPLING_TIME[0] 毎に呼び出される(リアルタイム空間なので処理は制御周期内に収めること))
		// リアルタイム制御ここから
		Interface::GetPosition(PositionRes);// [rad]位置応答の取得
		Interface::GetTorque(TorqueRes);	// [Nm] トルク応答の取得
		Interface::GetCurrent(CurrentRes);	// [A]  q軸電流応答の取得
		ScreenParams::GetOnlineSetVar(Kp);	// [-]  オンライン設定変数の読み込み (比例ゲイン手動調整用)
		t = pCF->GetTime();					// [s]  現在時刻の取得
		
		// ここに所望の制御コードを書くがよい！
		// (下記は単慣性系の1軸速度制御のコーディングの例)
		std::tie(wm, PositionRes[0]) = Motor1.GetResponses(iqref, tload);	// [rad/s,rad] モータシミュレータの計算
		tdis = DOb1.GetDistTorque(iqref, wm);	// [Nm] 外乱オブザーバの計算
		wmref = SquareWave(0.1, M_PI, t);		// [rad/s] 速度指令生成
		iqref = (wmref - wm)*Kp*Jm/Kt + tdis/Kt;// [A] P速度制御＋外乱オブザーバ加速度制御
		
		CurrentRef[0] = iqref;	// [A] 電流指令
		Interface::SetCurrent(CurrentRef);	// [A] 電流指令の出力
		ScreenParams::SetVarIndicator(wmref, wm, 0, 0, 0, 0, 0, 0, 0, 0);	// 任意変数インジケータ
		GraphPlot::SetVars(0, wmref, wm, 0, 0, 0, 0, 0, 0);	// グラフプロット
		GraphPlot::SetVars(1, 0, 0, 0, 0, 0, 0, 0, 0);	// グラフプロット
		GraphPlot::SetVars(2, 0, 0, 0, 0, 0, 0, 0, 0);	// グラフプロット
		GraphPlot::SetVars(3, 0, 0, 0, 0, 0, 0, 0, 0);	// グラフプロット
		pCF->ExpData.SetData(t, wmref, wm, 0, 0, 0, 0, 0, 0, 0);	// CSV/DATデータ保存変数
		// リアルタイム制御ここまで
	}
	if(pCF->CmdFlag == CTRL_EXIT){
		// 終了処理モード (ここは制御終了時に1度だけ呼び出される(非リアルタイム空間なので重い処理もOK))
		Interface::SetZeroCurrent();// 電流指令を零に設定
		Interface::ServoOFF();		// サーボOFF信号の送出
	}
}

//! @brief 制御用周期実行関数2
//! @param[in]	pCF	クラスメンバアクセス用ポインタ
void ControlFunctions::ControlFunction2(ControlFunctions* const pCF){
	// 制御用定数宣言
	
	// 制御用変数宣言
	
	if(pCF->CmdFlag == CTRL_INIT){
		// 初期化モード (ここは制御開始時/再開時に1度だけ呼び出される(非リアルタイム空間なので重い処理もOK))
	}
	if(pCF->CmdFlag == CTRL_LOOP){
		// 周期モード (ここは制御周期 SAMPLING_TIME[1] 毎に呼び出される(リアルタイム空間なので処理は制御周期内に収めること))
	}
	if(pCF->CmdFlag == CTRL_EXIT){
		// 終了処理モード (ここは制御終了時に1度だけ呼び出される(非リアルタイム空間なので重い処理もOK))
	}
}

//! @brief 制御用周期実行関数3
//! @param[in]	pCF	クラスメンバアクセス用ポインタ
void ControlFunctions::ControlFunction3(ControlFunctions* const pCF){
	// 制御用定数宣言
	
	// 制御用変数宣言
	
	if(pCF->CmdFlag == CTRL_INIT){
		// 初期化モード (ここは制御開始時/再開時に1度だけ呼び出される(非リアルタイム空間なので重い処理もOK))
	}
	if(pCF->CmdFlag == CTRL_LOOP){
		// 周期モード (ここは制御周期 SAMPLING_TIME[2] 毎に呼び出される(リアルタイム空間なので処理は制御周期内に収めること))
	}
	if(pCF->CmdFlag == CTRL_EXIT){
		// 終了処理モード (ここは制御終了時に1度だけ呼び出される(非リアルタイム空間なので重い処理もOK))
	}
}

//! @brief 制御用変数値を更新する関数
void ControlFunctions::UpdateControlValue(void){
	// ARCS画面パラメータに値を書き込む
	ScreenParams::SetNetworkLink(NetworkLink);		// ネットワークリンクフラグを書き込む
	ScreenParams::SetInitializing(Initializing);	// ロボット初期化フラグを書き込む
	ScreenParams::SetCurrentAndPosition(CurrentRef, PositionRes);	// 電流指令と位置応答を書き込む
}

