//! @file ConstParams.hh
//! @brief 定数値格納用クラス
//!        ARCSに必要な定数値を格納します。
//! @date 2019/04/01
//! @author Yuki YOKOKURA
//
// Copyright (C) 2011-2019 Yuki YOKOKURA
// This program is free software;
// you can redistribute it and/or modify it under the terms of the FreeBSD License.
// For details, see the License.txt file.

#ifndef CONSTPARAMS
#define CONSTPARAMS

#include "FrameGraphics.hh"
#include "DataStorage.hh"

// 注意！： const std::string は ConstParams.cc で定義すること。それ以外は constexpr として ConstParams.hh で定義。

namespace ARCS {	// ARCS名前空間
	//! @brief 定数値格納用クラス
	class ConstParams {
		public:
			// タイトルに表示させる制御系の名前(識別用に好きな名前を入力)
			static const std::string CTRLNAME;		// (60文字以内)
			
			// ARCS改訂番号(ARCSコード改変時にちゃんと変えること)
			static const std::string ARCS_REVISION;	// (16文字以内)
			
			// イベントログの設定
			static const std::string EVENTLOG_NAME;	// イベントログファイル名

			// ARCSユーザインタフェースの設定
			static constexpr unsigned long ARCS_UI_UPDATE_TIME = 15000;	// [us] ARCS画面描画の更新時間（ここの時間は厳密ではない）
			static constexpr unsigned int  ARCS_UI_CPU_NUMBER = 3;		// ARCS画面描画に割り当てるCPU番号（実時間スレッドとは別にすること）
			
			// 画面サイズの設定 (モニタ解像度に合うように設定すること)
			// 1024×600(WSVGA) の場合に下記をアンコメントすること
			//static constexpr int SCR_VERTICAL_MAX = 36;		// [文字] 画面の最大高さ文字数
			//static constexpr int SCR_HORIZONTAL_MAX = 127;	// [文字] 画面の最大幅文字数
			// 1024×768(XGA) の場合に下記をアンコメントすること
			static constexpr int SCR_VERTICAL_MAX = 47;		// [文字] 画面の最大高さ文字数
			static constexpr int SCR_HORIZONTAL_MAX = 127;	// [文字] 画面の最大幅文字数
			// 1280×1024(SXGA) の場合に下記をアンコメントすること
			//static constexpr int SCR_VERTICAL_MAX = 63;		// [文字] 画面の最大高さ文字数
			//static constexpr int SCR_HORIZONTAL_MAX = 159;	// [文字] 画面の最大幅文字数
			// 1920×1080(Full HD) の場合に下記をアンコメントすること
			//static constexpr int SCR_VERTICAL_MAX = 66;		// [文字] 画面の最大高さ文字数
			//static constexpr int SCR_HORIZONTAL_MAX = 239;	// [文字] 画面の最大幅文字数
			// それ以外の解像度の場合は各自で値を探すこと
			
			// 実験データファイルの設定
			static const std::string			DATA_NAME;								//!<	ファイル名
			static constexpr DataStorageType	DATA_TYPE = DataStorageType::FORMAT_CSV;//!<	CSV形式「FORMAT_CSV」DAT形式(TAB区切り)「FORMAT_DAT」
			static constexpr unsigned int		DATA_TIME =  10;						//!< [s] 保存時間
			static constexpr unsigned int		DATA_NUM  =  10;						//!< [-] 保存する変数の数
			static constexpr unsigned int		DATA_RATE =   1;						//!< [-] 保存データの間引き (計測データ数が割り切れる値にすること)
			
			// SCHED_FIFOリアルタイムスレッドの設定
			static constexpr unsigned int THREAD_MAX = 3;	//!< スレッド最大数（これ変えても ControlFunctions.cc は追随しないので注意）
			static constexpr unsigned int THREAD_NUM = 1;	//!< 動作させるスレッドの数 (最大数は THREAD_NUM_MAX 個まで)
			
			//! @brief 制御周期の設定
			static constexpr unsigned long SAMPLING_TIME[THREAD_MAX] = {
			//   s  m  u  n	制御周期は Ts[0] ≦ Ts[1] ≦ … ≦ Ts[THREAD_MAX] になるようにすること
				     100000,	// [ns] 制御用周期実行関数1 (スレッド1) 制御周期
				     250000,	// [ns] 制御用周期実行関数2 (スレッド2) 制御周期
				    1000000,	// [ns] 制御用周期実行関数2 (スレッド3) 制御周期
			};
			
			//! @brief 使用CPUコアの設定
			static constexpr unsigned int CPUCORE_NUMBER[THREAD_MAX] = {
				    1,	// [ns] 制御用周期実行関数1 (スレッド1) 使用するCPUコア番号
				    2,	// [ns] 制御用周期実行関数2 (スレッド2) 使用するCPUコア番号
				    3,	// [ns] 制御用周期実行関数2 (スレッド2) 使用するCPUコア番号
			};
			
			// 実験機アクチュエータの設定
			static constexpr unsigned int ACTUATOR_MAX = 16;//!< ARCSが対応しているアクチュエータの最大数
			static constexpr unsigned int ACTUATOR_NUM =  6;//!< 実験装置のアクチュエータの総数
			
			//! @brief アクチュエータタイプの定義
			enum ActType {
				LINEAR_MOTOR,	//!< リニアモータ
				ROTARY_MOTOR	//!< 回転モータ
			};
			
			//! @brief 実験機アクチュエータの種類の設定（リニアモータか回転モータかの設定）
			static constexpr ActType ACT_TYPE[ACTUATOR_MAX] = {
				ROTARY_MOTOR,	//  1番 アクチュエータ
				ROTARY_MOTOR,	//  2番 アクチュエータ
				ROTARY_MOTOR,	//  3番 アクチュエータ
				ROTARY_MOTOR,	//  4番 アクチュエータ
				ROTARY_MOTOR,	//  5番 アクチュエータ
				ROTARY_MOTOR,	//  6番 アクチュエータ
				ROTARY_MOTOR,	//  7番 アクチュエータ
				ROTARY_MOTOR,	//  8番 アクチュエータ
				ROTARY_MOTOR,	//  9番 アクチュエータ
				ROTARY_MOTOR,	// 10番 アクチュエータ
				ROTARY_MOTOR,	// 11番 アクチュエータ
				ROTARY_MOTOR,	// 12番 アクチュエータ
				ROTARY_MOTOR,	// 13番 アクチュエータ
				ROTARY_MOTOR,	// 14番 アクチュエータ
				ROTARY_MOTOR,	// 15番 アクチュエータ
				ROTARY_MOTOR,	// 16番 アクチュエータ
			};
			
			//! @brief トルク/推力定数の設定
			static constexpr double ACT_FORCE_TORQUE_CONST[ACTUATOR_MAX] = {
				1,	// [N/A]/[Nm/A]  1番 アクチュエータ
				1,	// [N/A]/[Nm/A]  2番 アクチュエータ
				1,	// [N/A]/[Nm/A]  3番 アクチュエータ
				1,	// [N/A]/[Nm/A]  4番 アクチュエータ
				1,	// [N/A]/[Nm/A]  5番 アクチュエータ
				1,	// [N/A]/[Nm/A]  6番 アクチュエータ
				1,	// [N/A]/[Nm/A]  7番 アクチュエータ
				1,	// [N/A]/[Nm/A]  8番 アクチュエータ
				1,	// [N/A]/[Nm/A]  9番 アクチュエータ
				1,	// [N/A]/[Nm/A] 10番 アクチュエータ
				1,	// [N/A]/[Nm/A] 11番 アクチュエータ
				1,	// [N/A]/[Nm/A] 12番 アクチュエータ
				1,	// [N/A]/[Nm/A] 13番 アクチュエータ
				1,	// [N/A]/[Nm/A] 14番 アクチュエータ
				1,	// [N/A]/[Nm/A] 15番 アクチュエータ
				1,	// [N/A]/[Nm/A] 16番 アクチュエータ
			};
			
			//! @brief 定格電流値の設定
			static constexpr double ACT_RATED_CURRENT[ACTUATOR_MAX] = {
				1,	// [A]  1番 アクチュエータ
				1,	// [A]  2番 アクチュエータ
				1,	// [A]  3番 アクチュエータ
				1,	// [A]  4番 アクチュエータ
				1,	// [A]  5番 アクチュエータ
				1,	// [A]  6番 アクチュエータ
				1,	// [A]  7番 アクチュエータ
				1,	// [A]  8番 アクチュエータ
				1,	// [A]  9番 アクチュエータ
				1,	// [A] 10番 アクチュエータ
				1,	// [A] 11番 アクチュエータ
				1,	// [A] 12番 アクチュエータ
				1,	// [A] 13番 アクチュエータ
				1,	// [A] 14番 アクチュエータ
				1,	// [A] 15番 アクチュエータ
				1,	// [A] 16番 アクチュエータ
			};
			
			//! @brief 瞬時最大許容電流値の設定
			static constexpr double ACT_MAX_CURRENT[ACTUATOR_MAX] = {
				3,	// [A]  1番 アクチュエータ
				3,	// [A]  2番 アクチュエータ
				3,	// [A]  3番 アクチュエータ
				3,	// [A]  4番 アクチュエータ
				3,	// [A]  5番 アクチュエータ
				3,	// [A]  6番 アクチュエータ
				3,	// [A]  7番 アクチュエータ
				3,	// [A]  8番 アクチュエータ
				3,	// [A]  9番 アクチュエータ
				3,	// [A] 10番 アクチュエータ
				3,	// [A] 11番 アクチュエータ
				3,	// [A] 12番 アクチュエータ
				3,	// [A] 13番 アクチュエータ
				3,	// [A] 14番 アクチュエータ
				3,	// [A] 15番 アクチュエータ
				3,	// [A] 16番 アクチュエータ
			};
			
			//! @biref 減速比の設定
			static constexpr double ACT_GEAR_RATIO[ACTUATOR_MAX] = {
				1,	//  1番アクチュエータ
				1,	//  2番アクチュエータ
				1,	//  3番アクチュエータ
				1,	//  4番アクチュエータ
				1,	//  5番アクチュエータ
				1,	//  6番アクチュエータ
				1,	//  7番アクチュエータ
				1,	//  8番アクチュエータ
				1,	//  9番アクチュエータ
				1,	// 10番アクチュエータ
				1,	// 11番アクチュエータ
				1,	// 12番アクチュエータ
				1,	// 13番アクチュエータ
				1,	// 14番アクチュエータ
				1,	// 15番アクチュエータ
				1,	// 16番アクチュエータ
			};
			
			// インタフェースボード関連の定数
			
			// デバッグプリントとデバッグインジケータの設定
			static constexpr bool DEBUG_PRINT_VISIBLE = false;	//!< デバッグプリント表示の有効/無効設定
			static constexpr bool DEBUG_INDIC_VISIBLE = false;	//!< デバッグインジケータ表示の有効/無効設定
			
			// 任意変数値表示の設定
			static constexpr unsigned int INDICVARS_MAX = 16;	//!< 表示変数最大数 (変更不可)
			static constexpr unsigned int INDICVARS_NUM = 10;	//!< 表示したい変数の数 (最大数 INDICVARS_MAX まで)
			static const std::string INDICVARS_FORMS[];			//!< 任意に表示したい変数値の表示形式 (printfの書式と同一)
			
			// オンライン設定変数の設定
			static constexpr unsigned int ONLINEVARS_MAX = 16;	//!< オンライン設定変数最大数 (変更不可)
			static constexpr unsigned int ONLINEVARS_NUM = 10;	//!< オンライン設定変数の数 (最大数 ONLINEVARS_MAX まで)
			
			// グラフプロットの共通設定
			static const std::string PLOT_FRAMEBUFF;			//!< フレームバッファ ファイルデスクリプタ
			static constexpr size_t PLOT_MAX = 16;				//!< グラフプロットの最大数 (変更不可)
			static constexpr size_t PLOT_NUM =  4;				//!< グラフプロットの数
			static constexpr double PLOT_RESET_TIME = 10;		//!< [s] プロットのリセット時間
			static constexpr double PLOT_TMIN = 0;				//!< [s] 横軸最小値
			static constexpr unsigned int PLOT_TGRID_NUM = 10;	//!< 横軸グリッドの分割数
			static const std::string PLOT_TFORMAT;				//!< 横軸書式
			static const std::string PLOT_TLABEL;				//!< 横軸ラベル
			static const std::array<std::string, PLOT_MAX> PLOT_FLABEL;	//!< 縦軸ラベル
			static const std::array<std::string, PLOT_MAX> PLOT_FFORMAT;//!< 縦軸書式
			static constexpr unsigned int PLOT_VAR_MAX = 8;		//!< プロット可能な変数の最大数 (変更不可)
			static const std::array<
				std::array<std::string, PLOT_VAR_MAX>, PLOT_MAX
			> PLOT_VAR_NAMES;	//!< 変数の名前
			static constexpr FrameGraphics::FGcolors PLOT_AXIS_COLOR = FrameGraphics::WHITE;	//!< 軸の色
			static constexpr FrameGraphics::FGcolors PLOT_GRID_COLOR = FrameGraphics::GRAY25;	//!< グリッドの色
			static constexpr FrameGraphics::FGcolors PLOT_BACK_COLOR = FrameGraphics::BLACK;	//!< 背景色
			static constexpr FrameGraphics::FGcolors PLOT_TEXT_COLOR = FrameGraphics::WHITE;	//!< 文字色
			
			//! @brief グラフ描画の有効/無効設定
			static constexpr std::array<bool, ConstParams::PLOT_MAX> PLOT_VISIBLE = {
				true,	// プロット0
				true,	// プロット1
				true,	// プロット2
				true,	// プロット3
				true,	// プロット4
				true,	// プロット5
				true,	// プロット6
				true,	// プロット7
				true,	// プロット8
				true,	// プロット9
				true,	// プロット10
				true,	// プロット11
				false,	// プロット12
				false,	// プロット13
				false,	// プロット14
				false,	// プロット15
			};
			
			//! @brief 線の色
			static constexpr std::array<FrameGraphics::FGcolors, PLOT_VAR_MAX> PLOT_VAR_COLORS = {
				FrameGraphics::CYAN,
				FrameGraphics::RED,
				FrameGraphics::GREEN,
				FrameGraphics::MAGENTA,
				FrameGraphics::YELLOW,
				FrameGraphics::ORANGE,
				FrameGraphics::WHITE,
				FrameGraphics::BLUE,
			};
			
			//! @brief プロットする変数の数 (≦PLOT_VAR_MAX)
			static constexpr std::array<unsigned int, PLOT_MAX> PLOT_VAR_NUM = {
				1,	// プロット0
				1,	// プロット1
				1,	// プロット2
				1,	// プロット3
				1,	// プロット4
				1,	// プロット5
				1,	// プロット6
				1,	// プロット7
				1,	// プロット8
				1,	// プロット9
				1,	// プロット10
				1,	// プロット11
				1,	// プロット12
				1,	// プロット13
				1,	// プロット14
				1,	// プロット15
			};
			
			//! @brief 縦軸最大値
			static constexpr std::array<double, PLOT_MAX> PLOT_FMAX	= {
				1.0,	// プロット0
				1.0,	// プロット1
				1.0,	// プロット2
				1.0,	// プロット3
				1.0,	// プロット4
				1.0,	// プロット5
				1.0,	// プロット6
				1.0,	// プロット7
				1.0,	// プロット8
				1.0,	// プロット9
				1.0,	// プロット10
				1.0,	// プロット11
				1.0,	// プロット12
				1.0,	// プロット13
				1.0,	// プロット14
				1.0,	// プロット15
			};
			
			//! @brief 縦軸最小値
			static constexpr std::array<double, PLOT_MAX> PLOT_FMIN = {
				-1.0,	// プロット0
				-1.0,	// プロット1
				-1.0,	// プロット2
				-1.0,	// プロット3
				-1.0,	// プロット4
				-1.0,	// プロット5
				-1.0,	// プロット6
				-1.0,	// プロット7
				-1.0,	// プロット8
				-1.0,	// プロット9
				-1.0,	// プロット10
				-1.0,	// プロット11
				-1.0,	// プロット12
				-1.0,	// プロット13
				-1.0,	// プロット14
				-1.0,	// プロット15
			};
			
			//! @brief 縦軸グリッドの分割数
			static constexpr std::array<unsigned int, PLOT_MAX> PLOT_FGRID_NUM = {
				4,	// プロット0
				4,	// プロット1
				4,	// プロット2
				4,	// プロット3
				4,	// プロット4
				4,	// プロット5
				4,	// プロット6
				4,	// プロット7
				4,	// プロット8
				4,	// プロット9
				4,	// プロット10
				4,	// プロット11
				4,	// プロット12
				4,	// プロット13
				4,	// プロット14
				4,	// プロット15
			};
			
			//! @brief [px] 左位置
			static constexpr std::array<int, ConstParams::PLOT_MAX> PLOT_LEFT = {
				305,	// プロット0
				305,	// プロット1
				305,	// プロット2
				305,	// プロット3
				305,	// プロット4
				305,	// プロット5
			   1015,	// プロット6
			   1015,	// プロット7
			   1015,	// プロット8
			   1015,	// プロット9
			   1015,	// プロット10
			   1015,	// プロット11
				  0,	// プロット12
				  0,	// プロット13
				  0,	// プロット14
				  0,	// プロット15
			};
			
			//! @brief [px] 上位置
			static constexpr std::array<int, ConstParams::PLOT_MAX> PLOT_TOP = {
				 97,	// プロット0
				250,	// プロット1
				403,	// プロット2
				556,	// プロット3
				709,	// プロット4
				862,	// プロット5
				 97,	// プロット6
				250,	// プロット7
				403,	// プロット8
				556,	// プロット9
				709,	// プロット10
				862,	// プロット11
				  0,	// プロット12
				  0,	// プロット13
				  0,	// プロット14
				  0,	// プロット15
			};
			
			//! @brief [px] 幅
			static constexpr std::array<int, ConstParams::PLOT_MAX> PLOT_WIDTH = {
				710,	// プロット0
				710,	// プロット1
				710,	// プロット2
				710,	// プロット3
				710,	// プロット4
				710,	// プロット5
				710,	// プロット6
				710,	// プロット7
				710,	// プロット8
				710,	// プロット9
				710,	// プロット10
				710,	// プロット11
				710,	// プロット12
				710,	// プロット13
				710,	// プロット14
				710,	// プロット15
			};
			
			//! @brief [px] 高さ
			static constexpr std::array<int, ConstParams::PLOT_MAX> PLOT_HEIGHT = {
				160,	// プロット0
				160,	// プロット1
				160,	// プロット2
				160,	// プロット3
				160,	// プロット4
				160,	// プロット5
				160,	// プロット6
				160,	// プロット7
				160,	// プロット8
				160,	// プロット9
				160,	// プロット10
				160,	// プロット11
				160,	// プロット12
				160,	// プロット13
				160,	// プロット14
				160,	// プロット15
			};
			
		private:
			ConstParams() = delete;	//!< コンストラクタ使用禁止
			~ConstParams() = delete;//!< デストラクタ使用禁止
			ConstParams(const ConstParams&) = delete;					//!< コピーコンストラクタ使用禁止
			const ConstParams& operator=(const ConstParams&) = delete;	//!< 代入演算子使用禁止
	};
}

#endif

