//! @file GraphPlot.cc
//! @brief グラフプロットクラス
//!
//! グラフを描画するクラス
//!
//! @date 2019/03/29
//! @author Yuki YOKOKURA
//
// Copyright (C) 2011-2019 Yuki YOKOKURA
// This program is free software;
// you can redistribute it and/or modify it under the terms of the FreeBSD License.
// For details, see the License.txt file.

#include <new>
#include "GraphPlot.hh"
#include "ConstParams.hh"
#include "ScreenParams.hh"
#include "FrameGraphics.hh"

using namespace ARCS;

//! @brief プロット描画変数の実体
std::array<
	std::array<
		double, ConstParams::PLOT_VAR_MAX
	>, ConstParams::PLOT_MAX
> GraphPlot::PlotVars = {{0}};

pthread_mutex_t GraphPlot::PlotVarsMutex = PTHREAD_MUTEX_INITIALIZER;	//!< プロット描画変数用のMutexの実体
size_t GraphPlot::PlotNumBuf = 0;										//!< プロット描画番号バッファの実体
std::array<double, ConstParams::PLOT_VAR_MAX> GraphPlot::VarsBuf = {0};	//!< プロット描画変数バッファの実体
size_t GraphPlot::VarsCount = 0;										//!< 再帰カウンタの実体

//! @brief コンストラクタ
GraphPlot::GraphPlot(std::string Device)
	: FG(FrameGraphics(Device.c_str())), Plot(), t_offset(0)
{
	PassedLog();
	
	// 「placement new」によるCuiPlotクラス配列の初期化
	for(size_t i = 0; i < ConstParams::PLOT_NUM; ++i){
		new(Plot.data() + i) CuiPlot(
			FG,
			ConstParams::PLOT_LEFT.at(i),
			ConstParams::PLOT_TOP.at(i),
			ConstParams::PLOT_WIDTH.at(i),
			ConstParams::PLOT_HEIGHT.at(i)
		);
	}
	
	// グラフプロットの設定
	for(size_t i = 0; i < ConstParams::PLOT_NUM; ++i){
		Plot.at(i).SetVisible(ConstParams::PLOT_VISIBLE.at(i));		// グラフ描画の有効/無効設定
		Plot.at(i).SetAxisColor(ConstParams::PLOT_AXIS_COLOR);		// 軸の色の設定
		Plot.at(i).SetGridColor(ConstParams::PLOT_GRID_COLOR);		// グリッドの色の設定
		Plot.at(i).SetBackColor(ConstParams::PLOT_BACK_COLOR);		// 背景色の設定
		Plot.at(i).SetTextColor(ConstParams::PLOT_TEXT_COLOR);		// 文字色の設定
		Plot.at(i).SetXrange(ConstParams::PLOT_TMIN, ConstParams::PLOT_RESET_TIME);			// X軸の範囲設定
		Plot.at(i).SetYrange(ConstParams::PLOT_FMIN.at(i), ConstParams::PLOT_FMAX.at(i));	// Y軸の範囲設定
		Plot.at(i).SetXgridNum(ConstParams::PLOT_TGRID_NUM);		// X軸グリッドの分割数の設定
		Plot.at(i).SetYgridNum(ConstParams::PLOT_FGRID_NUM.at(i));	// Y軸グリッドの分割数の設定
		Plot.at(i).SetXLabel(ConstParams::PLOT_TLABEL);				// X軸ラベルの設定
		Plot.at(i).SetYLabel(ConstParams::PLOT_FLABEL.at(i));		// Y軸ラベルの設定
		Plot.at(i).SetXform(ConstParams::PLOT_TFORMAT);				// X軸書式の設定
		Plot.at(i).SetYform(ConstParams::PLOT_FFORMAT.at(i));		// Y軸書式の設定
		Plot.at(i).SetVariables(
			ConstParams::PLOT_VAR_NUM.at(i),
			ConstParams::PLOT_VAR_NAMES.at(i).data(),
			ConstParams::PLOT_VAR_COLORS.data()
		);	// プロットする変数の数，名前，色の設定
	}
}

//! @brief ムーブコンストラクタ
GraphPlot::GraphPlot(GraphPlot&& r)
	: FG(std::move(r.FG)), Plot(std::move(r.Plot)), t_offset(std::move(r.t_offset))
{
	
}

//! @brief デストラクタ
GraphPlot::~GraphPlot(){
	PassedLog();
}

//! @brief プロット軸の描画
void GraphPlot::DrawAxes(void){
	for(size_t i = 0; i < ConstParams::PLOT_NUM; ++i){
		Plot.at(i).DrawAxis();	// 各プロットの軸の描画
	}
}

//! @brief 凡例の描画
void GraphPlot::DrawLegends(void){
	for(size_t i = 0; i < ConstParams::PLOT_NUM; ++i){
		Plot.at(i).DrawLegend();// 各プロットの凡例の描画
	}
}

//! @brief プロット波形の描画
void GraphPlot::DrawWaves(void){
	double t = ScreenParams::GetTime() - t_offset;	// グラフプロット用の横軸時間を計算
	
	if(ConstParams::PLOT_RESET_TIME < t){
		// リセット時間に到達したら、
		for(size_t i = 0; i < ConstParams::PLOT_NUM; ++i){
			Plot.at(i).ClearPlots();	// プロットを消去
			Plot.at(i).EndPlot();		// 逐次プロット終了処理
		}
		t_offset += ConstParams::PLOT_RESET_TIME;	// オフセット時間を加算
	}else{
		// リセット時間以内であれば、
		pthread_mutex_lock(&PlotVarsMutex);
		for(size_t i = 0; i < ConstParams::PLOT_NUM; ++i){
			Plot.at(i).PlotVariables(t, PlotVars.at(i).data());	// プロットの描画
		}
		pthread_mutex_unlock(&PlotVarsMutex);
	}
}

//! @brief プロットをリセットする関数
void GraphPlot::ResetWaves(void){
	for(size_t i = 0; i < ConstParams::PLOT_NUM; ++i){
		Plot.at(i).ClearPlots();	// プロットを消去
		Plot.at(i).EndPlot();		// 逐次プロット終了処理
	}
	t_offset = 0;	// オフセット時間もリセット
}

