//! @file ARCSthread.cc
//! @brief ARCSリアルタイムスレッド管理クラス
//!
//! リアルタイムスレッドの生成、開始、停止、破棄などの管理をします。
//!
//! @date 2019/01/20
//! @author Yuki YOKOKURA
//
// Copyright (C) 2011-2019 Yuki YOKOKURA
// This program is free software;
// you can redistribute it and/or modify it under the terms of the FreeBSD License.
// For details, see the License.txt file.

#include <unistd.h>
#include "ARCSthread.hh"
#include "ARCScommon.hh"
#include "ARCSeventlog.hh"
#include "ARCSassert.hh"
#include "ConstParams.hh"
#include "ScreenParams.hh"
#include "ControlFunctions.hh"
#include "SFmultthread.hh"

using namespace ARCS;

//! @brief コンストラクタ
ARCSthread::ARCSthread()
	: CtrlFuncs(), RealtimeThreads(), InfoGetThreadID()
{
	CtrlFuncs = new ControlFunctions();	// 制御用周期実行関数群の生成
	RealtimeThreads = new SFmultthread(	// リアルタイムマルチスレッドの生成
		ConstParams::THREAD_NUM,		// 動作させるスレッドの数
		(FuncPtrs)CtrlFuncs->pCFuncs,	// 制御用周期実行関数群への関数ポインタ
		CtrlFuncs,						// 関数ポインタ引数
		ConstParams::SAMPLING_TIME,		// 制御周期
		ConstParams::CPUCORE_NUMBER		// CPUコア番号
	);
}

//! @brief デストラクタ
ARCSthread::~ARCSthread(){
	delete RealtimeThreads; RealtimeThreads = nullptr;	// リアルタイムマルチスレッドの消去
	delete CtrlFuncs; CtrlFuncs = nullptr;				// 制御用実行関数群の消去
}

//! @ brief スレッドを開始する関数
void ARCSthread::Start(void){
	// 情報取得スレッド生成
	pthread_create(&InfoGetThreadID, NULL, (void*(*)(void*))InfoGetThread, this);			// 情報取得スレッド生成
	ARCScommon::SetCPUandPolicy(InfoGetThreadID, ConstParams::ARCS_UI_CPU_NUMBER, SCHED_RR);// CPUコアとポリシーの設定
	
	// 制御開始！
	PassedLog();
	CtrlFuncs->InitialProcess();	// 初期化モードの実行
	ARCSassert::SetRealtimeMode();	// ARCS用assertをリアルタイムモードに変更
	RealtimeThreads->Start();		// 実時間マルチスレッドの開始
}

//! @brief スレッドを停止する関数
void ARCSthread::Stop(void){
	// 制御停止！
	if(ARCSassert::IsEmergency() == false){
		// 正常終了時は，
		ARCSassert::SetNonRealtimeMode();	// ARCS用assertを非リアルタイムモードに変更
		RealtimeThreads->Stop();			// 実時間マルチスレッドの終了
	}else{
		// 緊急停止時は，
		RealtimeThreads->EmergencyStop();	// 実時間マルチスレッドの終了
	}
	PassedLog();
	CtrlFuncs->ExitProcess();			// 終了処理モードの実行
	
	// 情報取得スレッド終了
	pthread_cancel(InfoGetThreadID);
}

//! @brief スレッドをリセットする関数
void ARCSthread::Reset(void){
	// 一回削除
	delete RealtimeThreads; RealtimeThreads = nullptr;	// リアルタイムマルチスレッドの消去
	delete CtrlFuncs; CtrlFuncs = nullptr;				// 制御用実行関数群の消去
	
	// 再度生成
	CtrlFuncs = new ControlFunctions();	// 制御用周期実行関数群の生成
	RealtimeThreads = new SFmultthread(	// リアルタイムマルチスレッドの生成
		ConstParams::THREAD_NUM,		// 動作させるスレッドの数
		(FuncPtrs)CtrlFuncs->pCFuncs,	// 制御用周期実行関数群への関数ポインタ
		CtrlFuncs,						// 関数ポインタ引数
		ConstParams::SAMPLING_TIME,		// 制御周期
		ConstParams::CPUCORE_NUMBER		// CPUコア番号
	);
}

//! @brief 測定データを保存する関数
void ARCSthread::SaveDataFiles(void){
	CtrlFuncs->SaveDataFiles();	// データを保存
}

//! @brief 情報取得スレッド
//! @param[in]	クラスメンバアクセス用ポインタ
void ARCSthread::InfoGetThread(ARCSthread* const p){
	double Time = 0;													// [s] 時刻 (一番速いスレッド THREAD0 の時刻)
	std::array<double, ConstParams::THREAD_MAX> PeriodicTime = {0};		// [s] 計測された制御周期
	std::array<double, ConstParams::THREAD_MAX> ComputationTime = {0};	// [s] 計測された消費時間
	std::array<double, ConstParams::THREAD_MAX> MaxTime = {0};			// [s] 計測された制御周期の最大値
	std::array<double, ConstParams::THREAD_MAX> MinTime = {0};			// [s] 計測された制御周期の最小値
	
	// スレッドがキャンセルされるまで無限ループ
	while(1){
		// リアルタイムスレッドから時刻情報を取得
		// 以下は一時的な実装。そのうちSFthreadクラスをarrayに書き換える予定。
		p->RealtimeThreads->GetTimeValue(
			&Time,					// 時刻の取得
			PeriodicTime.data(),	// 制御周期の取得
			ComputationTime.data(),	// 消費時間の取得
			MaxTime.data(),			// 制御周期の最大値の取得
			MinTime.data()			// 制御周期の最小値の取得
		);
		
		// ARCS画面パラメータに格納
		ScreenParams::SetTime(Time);
		ScreenParams::SetTimeVars(PeriodicTime, ComputationTime, MaxTime, MinTime);
		
		// 制御用変数値を更新
		p->CtrlFuncs->UpdateControlValue();	// 制御用周期実行関数群クラス内部で，ARCS画面パラメータに書き込む/から読み込む
		
		usleep(ConstParams::ARCS_UI_UPDATE_TIME);	// 指定時間だけ待機
	}
}

